/*   -*- coding: utf-8 -*-  */

/**********************************************************************
*
*  module   : Ccsd
*  fichier  : CcsdHalAffiliationSetPays.c 
*  Auteur   : Jacques DUCLOY
*  Date     : fin 2013
*
*
* This file is common to Hal V2 and V3 versions.
*
***********************************************************************/

#include <stdio.h>     /* for printf */
#include <stdlib.h>    /* for exit */
/* #include <getopt.h> */
#include "SxPath.h"
#include "Buffer.h"
#include "WicriPaysFromEn.h"
#include "WicriFrance.h"
#include "TeiHandler.h"
#include "Utf8Converter.h"
#include "WicriPaysFromEn.h"

SxPathResult *teiPathAffiliation;
SxPathResult *teiPathAffiliationOld;

/* StrSearchTable *tablePays; */
SxmlNode *docInput;
SxmlNode *affiliation;
SxmlNode *listAffiliations;
int halVersion;  /* 2=V2 3=V3 */

char *isTutelleLabo(char *tutStr)
{
  char *posColon;
  posColon=strchr(tutStr,':');
  if (!posColon)return NULL;
  if (strncmp(tutStr,"CNRS : ", 7)==0)return tutStr;
  if (strncmp(tutStr,"CEA : ", 5)==0)return tutStr;
  if (strncmp(posColon-6, "[IRD] ",6)==0)return tutStr;
  if ((strncmp(tutStr, "Univ", 4)==0) && (strncmp(posColon, ": EA ",4)==0))return tutStr;
  return NULL;
}

void proceedTutelle(SxmlNode *halTutelle)
{
  
  SxmlNode *univNode;
  char *strTutelle;
  SxmlNode *tutelleNode;
  static Buffer *lcBuf=NULL;
  char *lcTutelle;
  
  if (!lcBuf) lcBuf=NewBuffer();
  strTutelle=SxmlLeafText(halTutelle);
  if (!strTutelle) return;

  lcTutelle=Utf8StringToSort(strTutelle);
  BufferStrcpy(lcBuf, lcTutelle);
  lcTutelle=BufferString(lcBuf);   /* as Utf8StringToSort() result is volatile */

  univNode=WicriUnivFrFromKey(strTutelle);
  if (!univNode)univNode=WicriUnivFrFromLcKey(lcTutelle);
  if ((univNode))
    {
      SxmlNode *placeName;
      SxmlNode *countryNode;
      SxmlNode *orgNode;
      if ((placeName=SxmlGetFirstChildByTagName(univNode, "placeName")))
	{
	  SxmlAppendChild (affiliation, SxmlClone(placeName));
	}
      if ((countryNode=SxmlGetFirstChildByTagName(univNode, "country")))
	{
	  TeiAffiliationSetCountry(affiliation, SxmlLeafText(countryNode));
	}
      if ((orgNode=SxmlGetFirstChildByTagName(univNode, "orgName")))
	{
	  while (orgNode)
	    {
	      SxmlAppendChild (affiliation, SxmlClone(orgNode));
	      orgNode=SxmlGetNextSiblingByTagName(orgNode, "orgName");
	    }
	}
      return;
    }

  /*  if ((strncmp(strTutelle, "CNRS : ", 7)==0)
      ||(strncmp(strTutelle, "CEA : ", 5)==0)) */
  if(isTutelleLabo(strTutelle))
    {
      SxmlNode *laboNode;
      if (SxmlHasAttribute(affiliation, "wicri:level", "5")) return;
      if ((laboNode=WicriFranceLaboFrFromKey(strTutelle)))
	{
	  TeiAffiliationUpdateFromOrgLab(affiliation, laboNode);
	  return;
	}
    }
  if ((tutelleNode=WicriFranceTutellesFrFromKey(strTutelle)))
    {
      SxmlNode *orgNode;
      SxmlNode *placeName;
      if ((orgNode=SxmlGetFirstChildByTagName(tutelleNode, "orgName")))
	{
	  while (orgNode)
	    {
	      SxmlAppendChild (affiliation, SxmlClone(orgNode));
	      orgNode=SxmlGetNextSiblingByTagName(orgNode, "orgName");
	    }
	}
      if ((placeName=SxmlGetFirstChildByTagName(tutelleNode, "placeName")))
	{
	  SxmlAppendChild (affiliation, SxmlClone(placeName));
	}
      return;
    }
  SxmlAppendChild(affiliation, SxmlLeafCreate("wicri:tutelleInc",  strTutelle));
}

SxmlNode *halTutelles;

void proceedAffiliationV2()
{
  if (halTutelles)
    {
      SxmlNode *halTutelle;
      SxmlReset (halTutelles);
      while ((halTutelle=SxmlNextNode(halTutelles)))
	{
	  proceedTutelle(halTutelle);
	}
    }
}

SxmlNode *halTutelleCNRS;
void proceedAffiliationV3()
{
  halTutelleCNRS=NULL;
  if (halTutelles)
    {
      SxmlNode *halTutelle;
      SxmlReset (halTutelles);
      while ((halTutelle=SxmlNextNode(halTutelles)))
	{
	  SxmlNode *orgTutelle;
	  SxmlNode *orgNameTutelle;
	  char *nameTutelle;
	  char *lcTutelle;
	  char *nameNodeTutelle;
	  SxmlNode *laboNode;
	  SxmlNode *univNode;
	  static Buffer *lcBuf=NULL;

	  if (!lcBuf) lcBuf=NewBuffer();
	  nameNodeTutelle=SxmlGetAttribute(halTutelle,"name");
	  orgTutelle=SxmlGetFirstChildByTagName(halTutelle, "org");
	  if (!orgTutelle)continue;
	  orgNameTutelle=SxmlGetFirstChildByTagName(orgTutelle, "orgName");
	  if (!orgNameTutelle)continue;
	  nameTutelle=SxmlLeafText(orgNameTutelle);
	  if (strcmp(nameTutelle, "CNRS")==0) 
	    {
	      if (!nameNodeTutelle) continue;
	      BufferStrcpy(lcBuf, "CNRS : ");
	      BufferStrcat(lcBuf, nameNodeTutelle);
	      laboNode=WicriFranceLaboFrFromKey(BufferString(lcBuf));
	      if (!laboNode)continue;
	      TeiAffiliationUpdateFromOrgLab(affiliation, laboNode);
	      continue;
	    }
	  univNode=WicriUnivFrFromKey(nameTutelle);
	  lcTutelle=Utf8StringToSort(nameTutelle);
	  BufferStrcpy(lcBuf, lcTutelle);
	  lcTutelle=BufferString(lcBuf);   /* as Utf8StringToSort() result is volatile */
	  if (!univNode)univNode=WicriUnivFrFromLcKey(lcTutelle);
	  if ((univNode))
	    {
	      SxmlNode *placeName;
	      SxmlNode *countryNode;
	      SxmlNode *orgNode;
	      if ((placeName=SxmlGetFirstChildByTagName(univNode, "placeName")))
		{
		  SxmlAppendChild (affiliation, SxmlClone(placeName));
		}
	      if ((countryNode=SxmlGetFirstChildByTagName(univNode, "country")))
		{
		  TeiAffiliationSetCountry(affiliation, SxmlLeafText(countryNode));
		}
	      if ((orgNode=SxmlGetFirstChildByTagName(univNode, "orgName")))
		{
		  while (orgNode)
		    {
		      SxmlAppendChild (affiliation, SxmlClone(orgNode));
		      orgNode=SxmlGetNextSiblingByTagName(orgNode, "orgName");
		    }
		}
	      continue;
	    }
	}
    }
}

void proceedAffiliation()
{
  SxmlNode *halAffiliation;
  if((halAffiliation=SxmlGetFirstChildByTagName(affiliation,"hal:affiliation")))
    {
      char     *halPays;
      SxmlNode *halDesc;
      SxmlNode *halAddress;
      SxmlNode *halCountry;
      halPays=NULL;
      if (halVersion==2)
	{
	  SxmlNode *halPaysNode;
	  if(( halPaysNode=SxmlGetFirstChildByTagName(halAffiliation,"pays")))
	    halPays=SxmlLeafText(halPaysNode);
	}
      else
	{
	  if ((halDesc=SxmlGetFirstChildByTagName(halAffiliation,"desc")))
	    {
	      if ((halAddress=SxmlGetFirstChildByTagName(halDesc,"address")))
		{
		  if ((halCountry=SxmlGetFirstChildByTagName(halAddress,"country")))
		    halPays=SxmlGetAttribute(halCountry,"key");
		}
	    }
	}
      if (halPays)
	{
	  char *pays;
	  if ((pays=WicriPaysFromIsoC2((halPays))))
	    {
	      TeiAffiliationSetCountry(affiliation, pays);
	      SxmlSetAttribute(affiliation, "wicri:level", "1");
	    }
	  else  SxmlAppendChild(affiliation,SxmlLeafCreate("wicri:noCountry", halPays));
	}

      halTutelles=SxmlGetFirstChildByTagName(halAffiliation,"tutelles");
      if (halVersion==2)proceedAffiliationV2();
      else proceedAffiliationV3();
    }
}

int main()
{
 teiPathAffiliationOld=SxPathSetCompile("TEI/teiHeader/fileDesc/titleStmt/author/affiliation");
 teiPathAffiliation=SxPathSetCompile("TEI/teiHeader/fileDesc/sourceDesc/biblStruct/analytic/author/affiliation");
  while ((docInput=SxmlInputNextDocumentElement()))
    {      
      SxmlNode *halRoot;
      char *attApi;

      if (!(SxmlNodeHasName(docInput,"record")))continue;
      halRoot=SxmlGetFirstChildByTagName(docInput,"hal");
      attApi=SxmlGetAttribute(halRoot,"api");
      if (strcmp(attApi, "V3")==0)halVersion=3;
      else halVersion=2;

      if ((listAffiliations=SxPathSetResultListCreate(teiPathAffiliation, docInput)))
	{
	  while((affiliation=SxmlNodeListNextNode(listAffiliations)))
	    {
	      proceedAffiliation();
	    }
	}
      if ((listAffiliations=SxPathSetResultListCreate(teiPathAffiliationOld, docInput)))
	{
	  while((affiliation=SxmlNodeListNextNode(listAffiliations)))
	    {
	      proceedAffiliation();
	    }
	}
      SxmlPrint (SxmlInputRecord);
    }
   exit(EXIT_SUCCESS);
}
