/**********************************************************************
*
*  module   : Istex
*  fichier  : IstexCatXml.c
*  Auteur   : Jacques DUCLOY
*  Date     : Février 2015
*
***********************************************************************
* 
*     Copyleft
*
***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "Hfd.h"

int istexC2;

SxmlParser *parserFileIstex;

char* IstexBufferFgetsUpToEOF(Buffer *b, FILE *s)
{
  int c; /* char c; */
  int c1;
  int beginLine;
  beginLine=1;
  while ((c=getc(s))!=EOF)
    {
      switch(c)
	{
	case '\015':
	  break;
	case '\n':
	  beginLine=1;
	  break;
	case ' ':
	case '\t':
	  if (beginLine==0) BufferCatChar(b,c);
	  break;
	case '<':
	  if ((c1=getc(s))!=EOF)
	    {
	      switch(c1)
		{
		case ' ':
		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
		  BufferEndString(b);
		  BufferStrcat(b,"&lt;");
		  BufferCatChar(b,c1);
		  break;
		default:
		  BufferCatChar(b,c);
		  BufferCatChar(b,c1);
		  beginLine=0;
		  break;
		}
	    }
	  else
	    {
	      BufferEndString(b);
	      BufferStrcat(b,"&lt;");
	      return b->buf;
	    }
	  break;
	default:
	  if (beginLine==1) BufferCatChar(b,' ');
	  beginLine=0;
	  BufferCatChar(b,c);
	  break;
	}
    }
  BufferEndString(b);
  return (b->buf);
}

int seekLt(FILE *inputFile)
{
  int c;
  while ((c=getc(inputFile))!=EOF)
    {
      if(c=='<') return c;
    }
  return ('\0');
}

void IstexCopyFile(FILE *inputFile, 
		 HfdStreamIterator *listInputStream)
{
  Buffer *bufFile=NULL;
  int codeDOM;
  if (!bufFile) bufFile=NewBuffer();
  else BufferReset(bufFile);
  
  printf("<istex>");
  printf("<istex:file>%s</istex:file>", HfdStreamIteratorCurrentFileName(listInputStream));
  printf("<istex:dtd>");
  IstexBufferFgetsUpToEOF(bufFile, inputFile);
  SxmlParserInit(parserFileIstex, BufferString(bufFile));
  while ((codeDOM=SxmlParserNextDomType(parserFileIstex)))
    {
      if (codeDOM==SXML_NODE_PROCESSING_INSTRUCTION)
	{
	  SxmlNode *nodePI;
	  nodePI=SxmlParserGetProcessingInstruction(parserFileIstex);
	  printf("<isxtex:xml %s/>", SxmlNodeValue(nodePI));
	  SxmlFree(nodePI);
	  continue;
	}
      else if (codeDOM==SXML_NODE_DOCUMENT_TYPE)
	{
	  SxmlNode *nodeDtd;
	  SxmlNode *outputDocType;
	  SxmlNode *dtdItem;
	  nodeDtd=SxmlParserGetDocType(parserFileIstex);
	  outputDocType=SxmlClone(nodeDtd);
	  SxmlSetNodeType(outputDocType, SXML_NODE_ELEMENT);
	  SxmlSetNodeName(outputDocType, strdup ("istex:docType"));
	  SxmlSetAttribute(outputDocType, "name", SxmlNodeName(nodeDtd));
	  SxmlReset(outputDocType);
	  while ((dtdItem=SxmlNextNode(outputDocType)))
	    {
	      char *n1;
	      n1=SxmlNodeName(dtdItem);
	      SxmlSetAttribute(dtdItem, "name", n1);
	      SxmlSetNodeType(dtdItem, SXML_NODE_ELEMENT);
	      SxmlSetNodeName(dtdItem, strdup ("istex:entity"));
	    }
	  SxmlPrint(outputDocType);
	  SxmlFree(nodeDtd);
	  SxmlFree(outputDocType);
	  continue;
	}
      break;
    }
  printf("</istex:dtd>");
  SxmlParserSkipSeparator(parserFileIstex);
  printf("<istex:document>");
  printf("%s", SxmlParserContextStr(parserFileIstex));
  printf("</istex:document>");
  printf("</istex>\n");
  return ;
}

int main (int argc, char **argv)
{

  FILE              *inputFile;
  HfdStreamIterator *listInputStream;

  parserFileIstex=SxmlParserCreate();

 listInputStream=HfdStreamIteratorCreate('a');
  HfdStreamIteratorSetStreams(listInputStream, argv+1);
  
  while ((inputFile=HfdNextStream(listInputStream)))
    {
      IstexCopyFile(inputFile, listInputStream);
    }

  return 0;
}
