/*   -*- coding: utf-8 -*-  */

/*
 IstexGetCorpus
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "Istex.h"

#include "CharSetBuffer.h"

extern char *optarg;
extern int   optind;
int getopt();

Buffer *bufTag;
SxmlNode *resultDoc;
SxmlNode *hitsElem;
int outputMode;
int outputType;
int allOption;
int modsOption;
char *typeMeta;
char *typeFullText;
char *jsonOutput;
int   json2sxml;

int IstexApiBlocSize;
int IstexApiListSize;
int IstexTotalHits;

SxmlNode *computeHitsElem(SxmlNode *resultDoc, char *query, char *edParam, int fromIntParam, int sizeIntParam)
{
  SxmlNode *hitsElem;

  if  (sizeIntParam<IstexApiBlocSize)
    {
      hitsElem=SxmlGetFirstChildByTagName(resultDoc, "hits");
    }
  else
    {
      int totalDoc;
      int localFrom;
      SxmlNode *globalList;
      SxmlNode *subList;
      
      localFrom=fromIntParam;
      globalList=IstexGetQueryResultSxml(query, edParam, localFrom , IstexApiBlocSize, "metadata", NULL);
      hitsElem=SxmlGetFirstChildByTagName(globalList, "hits");
      if ((IstexTraceLevel > 1))
	{
	  SxmlPrint(hitsElem);
	  putchar('\n');
	}
      localFrom+= IstexApiBlocSize;
      totalDoc=IstexApiBlocSize;
      while ((totalDoc<sizeIntParam))
	{
	  SxmlNode *item;
	  SxmlNode *subListHits;
	  int localSize;
	  
	  localSize=sizeIntParam-totalDoc;
	  if (localSize>IstexApiBlocSize) localSize=IstexApiBlocSize;
	  subList=IstexGetQueryResultSxml(query, edParam, localFrom , localSize, "metadata", NULL);
	  subListHits=SxmlGetFirstChildByTagName(subList, "hits");
	  totalDoc+=IstexApiBlocSize;
	  localFrom+= IstexApiBlocSize;
	  SxmlReset(subList);
	  while ((item=SxmlNextNode(subListHits)))
	    {
	      SxmlAppendChild (hitsElem, SxmlClone(item));
	    }
	  SxmlFree(subList);
	}
    }
  return hitsElem;
}

SxmlNode *getAndPrintOneFullDocFromQuery(char *query, char *edParam, int fromParam)
{
  SxmlNode *starResult;
  static SxmlNode *outputDoc=NULL;
  
  starResult=IstexGetQueryResultSxml(query, edParam, fromParam ,1 , "*", NULL);
  if (outputDoc)SxmlFree(outputDoc);
  switch(outputType)
  {
  case 'M':
    outputDoc=IstexGetFullDocFromJson(starResult, 0);
    break;
  case 'a':
    outputDoc=IstexGetFullDocFromJson(starResult, 1);
    break;
  case 'A':
    outputDoc=IstexGetFullDocFromJson(starResult, 2);
    break;
  default:
    ExceptSetError("IstexGetCorpus", "OP", "Iternal problem","","",1);
    exit(EXIT_FAILURE);
  } 
  SxmlPrint (outputDoc);
  SxmlFree(starResult);
  putchar ('\n');
  return outputDoc;
}

  
int main (int argc, char **argv)
{
  int cOption;
  
  char *query;
  char *sizeParam;
  int sizeIntParam;
  char *fromParam;
  int fromIntParam;
  int listIdOption;
  char *edParam;
  char *idParam;
  char *tokenParam;
  char *passParam;

  bufTag=NewBuffer();
  
  sizeParam=NULL;
  fromParam=NULL;
  edParam=NULL;
  typeMeta=NULL;
  typeFullText=NULL;
  idParam=NULL;
  passParam=NULL;
  tokenParam=NULL;
  outputMode =1;  /* just print result */
  sizeIntParam=10;
  fromIntParam=0;
  IstexApiBlocSize=100;
  IstexApiListSize=5000;
  allOption=0;
  modsOption=0;
  IstexInit();
  jsonOutput=NULL;
  json2sxml=0;
  outputType=0;
  listIdOption=0;

  while((cOption=getopt(argc,argv,"aAb:e:f:i:j:J:lmMo:O:s:p:q:t:u:xT:"))!=EOF)
    {switch (cOption)
	{
	case 'a':
	  allOption=1;
	  outputMode=3;
	  outputType='a';
	  break;
	case 'A':
	  allOption=2;
	  outputMode=3;
	  outputType='A';
	  break;
	case 'M':
	  allOption=1;
	  modsOption=1;
	  outputMode=3;
	  outputType='M';
	  break;
	case 'b':
	  IstexApiBlocSize=atoi(optarg);
	  break;
	case 'i':
	  idParam=optarg;
	  break;
	case 'j':
	  outputMode=1;
	  /*
	  if (strcmp(optarg, "1")==0) jsonOutput="*";
	  else jsonOutput=optarg;
	  */
	  jsonOutput=optarg;
	  json2sxml=0;
	  break;
	case 'J':
	  outputMode=1;
	  /*
	  if (strcmp(optarg, "1")==0) jsonOutput="*";
	  else if (strcmp(optarg, "0")==0) jsonOutput=NULL;
	  else jsonOutput=optarg;
	  */
	  jsonOutput=optarg;
	  json2sxml=1;
	  break;
	case 'l':  /* list id */
	  listIdOption=1;
	  break;
	case 'p':
	  passParam=optarg;
	  break;
	case 'q':
	  query=optarg;
	  break;
	case 't':
	  tokenParam=optarg;
	  break;
	case 'o':
	  typeMeta=optarg;
	  outputMode=4;
	  break;
	case 'O':
	  typeFullText=optarg;
	  outputMode=4;
	  break;
	case 'T':
	  IstexTraceLevel=atoi(optarg);
	  break;
	case 'm':   /* list url mods only */
	  outputMode=3;
	  break;
	case 'x':   /* result in xml coding */
	  outputMode=2;
	  break;
	case 's':
	  sizeParam=optarg;
	  sizeIntParam=atoi(optarg);
	  break;
	case 'u':
	  IstexRootUrl=optarg;
	  break;
	case 'e':
	  edParam=optarg;
	  break;
	case 'f':
	  fromParam=optarg;
	  fromIntParam=atoi(optarg);
	  break;
	}
    }

  if (tokenParam)IstexSetAuthToken(tokenParam);
  if (idParam&&passParam)IstexSetIdPass(idParam, passParam);

  if (listIdOption==1)
    {
      char *resultStr;
      SxmlNode *resultSxml;
      if (sizeIntParam <IstexApiListSize)
	{
	  resultStr=IstexGetQueryResult(query, edParam, fromIntParam, sizeIntParam, NULL, NULL);
	  if (resultStr)
	    {
	      SxmlNode *hitsNode;
	      SxmlNode *jsonItem;
	   
	      resultSxml=JsonToSxml(resultStr);
	      hitsNode=SxmlGetFirstChildByTagName(resultSxml,"hits");
	      if (hitsNode)
		{
		  while ((jsonItem=SxmlNextNode(hitsNode)))
		    {
		      SxmlNode *idNode;
		      idNode=SxmlGetFirstChildByTagName(jsonItem,"id");
		      printf("%s\n",SxmlLeafText(idNode));
		    }
		}
	    }

	}
      IstexEnd();
      exit(EXIT_SUCCESS);
    }
  if (outputMode==1)
    {
      int from;
      int size;
      char *resultStr;
      if (sizeIntParam<IstexApiBlocSize)
	{
	  from=fromIntParam;
	  size=sizeIntParam;
	}
      else
	{          /* to be improved */
	  from=0;
	  size=1;
	}
      resultStr=IstexGetQueryResult(query, edParam, fromIntParam, sizeIntParam, jsonOutput, NULL);
      if (json2sxml==0) printf("%s\n", resultStr);
      else 
	{
	  SxmlPrint(JsonToSxml(resultStr));
	  putchar('\n');
	}
      IstexEnd();
      exit(EXIT_SUCCESS);
    }

  if (sizeIntParam<IstexApiBlocSize)
    {
      if (typeMeta) resultDoc=IstexGetQueryResultSxml(query, edParam, fromIntParam, sizeIntParam, "metadata", NULL);
      else resultDoc=IstexGetQueryResultSxml(query, edParam, fromIntParam, sizeIntParam, "fulltext", NULL);
    }
  else
    {
      resultDoc=IstexGetQueryResultSxml(query, edParam, 0, 1,"metadata", NULL);
    }
  if (!resultDoc)
    {
      ExceptSetError("IstexGetCorpus", "XM", "Parsing result problem","","",1);
      exit(EXIT_FAILURE);
    }
  if ((outputMode==2)||(IstexTraceLevel > 0))
    {
      SxmlPrint(resultDoc);
      putchar('\n');
    }
  if (outputMode==2)
    {
      IstexEnd();
      return 0;
    }
  /*                                                        outputMode 3 or 4 from this point */

  IstexTotalHits=atoi(SxmlLeafText(SxmlGetFirstChildByTagName(resultDoc, "total")));

  if (allOption >= 1)
    {
      int toPrint;
      int from;
      if (fromIntParam > 0)
	{
	  int maxToPrint;
	  maxToPrint=IstexTotalHits-fromIntParam;
	  if (maxToPrint < sizeIntParam) toPrint=maxToPrint;
	  else toPrint=sizeIntParam;
	}
      else
	{
	  if (IstexTotalHits < sizeIntParam) toPrint=IstexTotalHits;
	  else toPrint=sizeIntParam;
	}
      from=fromIntParam;
      while ((toPrint--)>0)getAndPrintOneFullDocFromQuery(query,edParam, from++);

    }
  else
    {
      hitsElem=computeHitsElem(resultDoc, query, edParam, fromIntParam, sizeIntParam);
      
      if (hitsElem)
	{
	  SxmlNode *hitsItem;
	  SxmlReset (hitsElem);
	  while ((hitsItem=SxmlNextNode(hitsElem)))
	    {
	      if (typeMeta) IstexGetAndPrintMetadataItem(hitsItem, outputMode, typeMeta);
	      else IstexGetAndPrintFullTextItem(hitsItem, typeFullText);
	    }
	}
    }
  
  IstexEnd();
  exit(EXIT_SUCCESS);
}
