/*   -*- coding: utf-8 -*-  */

/**********************************************************************
*
*  module   : Istex
*  comande  : IstexModsToTei
*  fichier  : IstexModsToTei.c
*  Auteur   : Jacques DUCLOY
*  Date     : 2013
*
***********************************************************************/

#include <stdio.h>     /* for printf */
#include <stdlib.h>    /* for exit */
#include <ctype.h>
#include <getopt.h>
#include "TeiHandler.h"
#include "Buffer.h"
#include "Utf8Converter.h"
#include "IstexRecord.h"

SxmlNode *docInput;
SxmlNode *corpusRecord;
SxmlNode *teiRoot;
SxmlNode *modsRoot;

SxmlNode *istexRelatedItem;

Buffer *bufSort;
Buffer *bufUniq;

void computeTitle()
{
  SxmlNode *title;
  SxmlNode *titleInfo;
  char *lang;
  if ((titleInfo=SxmlGetFirstChildByTagName(modsRoot,"titleInfo")))
    {
      if ((title=SxmlGetFirstChildByTagName(titleInfo,"title")))
	{
	  SxmlNode *titleField;
	  titleField=SxmlClone(title);
	  SxmlAppendChild(Tei_titleStmt,titleField);
	  lang=SxmlGetAttribute(titleInfo,"lang");
	  if (lang)                             /* to be improved */
	    {
	      if (strncmp(lang, "en",2)==0)SxmlSetAttribute(titleField, "xml:lang", "en");
	      else SxmlSetAttribute(titleField, "xml:lang", lang);
	    }
	  SxmlAppendChild(Tei_analytic, SxmlClone(titleField));
	}
    }
}

void computeDate()
{
  SxmlNode *originInfo;
  SxmlNode *relatedItem;
  SxmlNode *copyrightDate;
  SxmlNode *targetDate;
  char *dateStr;
  SxmlNode *teiDate;

  targetDate=NULL;

  if ((originInfo=SxmlGetFirstChildByTagName(modsRoot,"originInfo")))
    {
      copyrightDate=SxmlGetFirstChildByTagName(originInfo,"copyrightDate");
      targetDate=copyrightDate;
    }
  else copyrightDate=NULL;

  if (!copyrightDate)
    {
      if ((relatedItem=SxmlGetFirstChildByTagName(modsRoot,"relatedItem"))
	  && (originInfo=SxmlGetFirstChildByTagName(relatedItem,"originInfo")))
	{
	  copyrightDate=SxmlGetFirstChildByTagName(originInfo,"copyrightDate");
	  targetDate=copyrightDate;
	}
    }

  if ((!targetDate)&&(originInfo))
    {
      targetDate=SxmlGetFirstChildByTagName(originInfo,"dateCreated");
    }
  if (targetDate)
    {
      dateStr=SxmlLeafText(targetDate);
      SxmlAppendChild(Tei_publicationStmt, teiDate=SxmlLeafCreate("date", dateStr));
      SxmlSetAttribute(teiDate,"when", dateStr);
    }
  else 
    {
      SxmlAppendChild(Tei_publicationStmt, teiDate=SxmlLeafCreate("date", "????"));
      SxmlSetAttribute(teiDate,"when", "????");
    }
}

char* computeId()
{
  SxmlNode *recordInfo;
  SxmlNode *recordIdentifierNode;
  SxmlNode *istexNode;
  char *recordIdentifier;

  recordIdentifier=NULL;
  istexNode=SxmlGetFirstChildTagAtt (modsRoot, "identifier", "type", "istex"); 
  if (istexNode)
    {
      recordIdentifier=SxmlLeafText(istexNode);
      TeiSetIdnoRbid("ISTEX", recordIdentifier);
      return recordIdentifier;
    }
  if ((recordInfo=SxmlGetFirstChildByTagName(modsRoot,"recordInfo")))
    {
      if((recordIdentifierNode=SxmlGetFirstChildByTagName(recordInfo,"recordIdentifier")))
	{
	  recordIdentifier=SxmlLeafText(recordIdentifierNode);
	  TeiSetIdnoRbid("ISTEX", recordIdentifier);
	}
    }
  return recordIdentifier;
}

void computeDoi()
{
  SxmlNode *doiNode;
  doiNode=SxmlGetFirstChildTagAtt (modsRoot, "identifier", "type", "DOI"); 
  if (!doiNode)doiNode=SxmlGetFirstChildTagAtt (modsRoot, "identifier", "type", "doi"); 
  if (doiNode)  TeiAppendIdno("doi", SxmlLeafText(doiNode));
}

void computeAbstract()
{
  SxmlNode *absNode;
  absNode=SxmlGetFirstChildByTagName(modsRoot,"abstract");
  if (!absNode) return;
  TeiHandlerStoreAbstract(absNode, SxmlGetAttribute(absNode, "lang"));
}

void computeRelated()
{
  SxmlNode *doiNode;
  if (!istexRelatedItem)return;
  doiNode=SxmlGetFirstChildTagAtt (istexRelatedItem, "identifier", "type", "doi"); 
  if (!doiNode)return;
  TeiHandlerSetSeriesDoi(SxmlLeafText(doiNode));
}

void computeISSN()
{
  SxmlNode *issnNode;
  SxmlNode *titleInfo;
  if (!istexRelatedItem)return;
  issnNode=SxmlGetFirstChildTagAtt (istexRelatedItem, "identifier", "type", "ISSN"); 
  while (issnNode)
    {
      char *issnVal;
      issnVal=SxmlLeafText(issnNode);
      if (strncmp(issnVal, "Electronic:",11)==0)TeiHandlerSetEIssn(issnVal+12);
      else TeiHandlerSetIssn(issnVal);
      issnNode=SxmlGetNextSiblingTagAtt (issnNode, "identifier", "type", "ISSN"); 
    }
  titleInfo=SxmlGetFirstChildByTagName(istexRelatedItem, "titleInfo");
  if (titleInfo)
    {
      SxmlNode *titleNode;
      titleNode=SxmlGetFirstChildByTagName(titleInfo, "title");
      if (titleNode)
	{
	  TeiHandlerSetSeriesTitle(SxmlLeafText(titleNode), "j", NULL); /* to be improved ???*/
	}
    }
}

void computeUrl()
{
  SxmlNode *fulltextNode;
  SxmlNode *jsonItem;
  fulltextNode=SxmlGetFirstChildByTagName(docInput,"fulltext");
  if (!fulltextNode) return;
  SxmlReset(fulltextNode);
  while ((jsonItem=SxmlNextNode(fulltextNode)))
    {
      SxmlNode *extension;
      SxmlNode *uri;
      extension=SxmlGetFirstChildByTagName(jsonItem, "extension");
      if (!extension)continue;
      uri=SxmlGetFirstChildByTagName(jsonItem, "uri");
      if (!uri)continue;
      if (SxmlIsLeaf(extension)&&(strcmp(SxmlLeafText(extension),"pdf")==0))
	{
	  TeiAppendIdno("url", SxmlLeafText(uri));
	}
      
    }
  
}

void computeTopics()
{
    SxmlNode *subjectNode;
    SxmlNode *topicNode;
    char *lang;
    subjectNode=SxmlGetFirstChildByTagName(modsRoot,"subject");
    while (subjectNode)
      {
	lang=SxmlGetAttribute(subjectNode, "lang");
	if ((lang)&&(strcmp(lang, "eng")==0))
	  {
	    topicNode=SxmlGetFirstChildByTagName(subjectNode,"topic");
	    while (topicNode)
	      {
		static Buffer *bufKw=NULL;
		char *newKw;
		if (!bufKw)bufKw=NewBuffer();
		BufferStrcpy (bufKw, newKw=SxmlLeafText(topicNode));
		if (islower(newKw[0]))newKw[0]=toupper(newKw[0]);
		TeiKeywordsPutTerm("KwdEn", NULL, NULL, "en", newKw);
		topicNode=SxmlGetNextSiblingByTagName(topicNode, "topic");
	      }
	  }
	subjectNode=SxmlGetNextSiblingByTagName(subjectNode, "subject");
      }
}

int isAuthor(SxmlNode *nameNode)   /* return an integer between 0 and 100 */
{
  SxmlNode *role;
  SxmlNode *roleTerm;
  role=SxmlGetFirstChildByTagName(nameNode,"role");
  if (!role) return 90;    /* author implicit ????? */
  roleTerm=SxmlGetFirstChildByTagName(role,"roleTerm");
  if (strcmp(SxmlLeafText(roleTerm),"author")==0) return 100;
  if (strcmp(SxmlLeafText(roleTerm),"editor")==0) return 0;
  return 80;
}

void setTeiAuthors()
{
  SxmlNode *nameNode;
  SxmlNode *teiAuthor;
  SxmlNode *teiName;
  static Buffer *bufGiven=NULL;
  static Buffer *bufName=NULL;
  static Buffer *bufFamily=NULL;
  if (!bufGiven)bufGiven=NewBuffer();
  if (!bufFamily)bufFamily=NewBuffer();
  if (!bufName)bufName=NewBuffer();
  nameNode=SxmlGetFirstChildByTagName(modsRoot,"name");
  while (nameNode)
    {
      if (SxmlHasAttribute(nameNode, "type", "personal"))
	{
	  SxmlNode *namePart;
	  SxmlNode *istexAffiliation;
	  int isAut;
	  char *given;
	  char *family;
	  given=NULL;
	  family=NULL;
	  /* if ((role=SxmlGetFirstChildByTagName(nameNode,"role"))
	      &&(roleTerm=SxmlGetFirstChildByTagName(role,"roleTerm"))
	      &&(strcmp(SxmlLeafText(roleTerm),"author")==0)
	      )
	  */
	  isAut=isAuthor(nameNode);
	  if (isAut>0)
	    {
	      namePart=SxmlGetFirstChildByTagName(nameNode,"namePart");
	      while (namePart)
		{
		  if (SxmlHasAttribute(namePart,"type","given"))
		    {
		      given=SxmlLeafText(namePart);
		      if (given)
			{
			  BufferStrcpy(bufGiven,given);

			  while (BufferTailCmp(bufGiven," ")==0) BufferTailCut(bufGiven,1);
			}
		    }
		  else if (SxmlHasAttribute(namePart,"type","family")) family=SxmlLeafText(namePart);
		  else family=SxmlLeafText(namePart);
		  namePart=SxmlGetNextSiblingByTagName(namePart,"namePart");
		}
	      if ((given) &&family)
		{
		  BufferStrcpy (bufName, BufferString(bufGiven));  
		  BufferStrcat (bufName, " ");  
		  BufferStrcat (bufName, family);
		  BufferStrcpy(bufSort, Utf8NameToWikiSort(family));
		  BufferStrcpy(bufUniq, Utf8NameToWikiSort(family));
		   BufferStrcat(bufSort, ", ");
		   BufferStrcat(bufUniq, " ");
		   BufferStrcat(bufSort, Utf8NameToWikiSort(given));
		   BufferCatChar(bufUniq, Utf8NameToWikiSort(given)[0]);
		   BufferCatChar(bufUniq, '\0');
		  teiAuthor=SxmlElementCreate("author");
		  if(isAut<100)
		    {
		      static char *bNum=NULL;
		      if (!bNum) bNum=malloc(4);
		      sprintf(bNum,"%d%c", isAut,'%');
		      SxmlSetAttribute(teiAuthor, "wicri:is", bNum);
		    }
		  SxmlAppendChild(Tei_titleStmt,teiAuthor);
		  SxmlAppendChild(teiAuthor, teiName=SxmlLeafCreate("name", BufferString(bufName)));

		  SxmlSetAttribute(teiName,  "sortKey", BufferString(bufSort));
		  SxmlSetAttribute(teiName,  "uniqKey", BufferString(bufUniq));
		  istexAffiliation=SxmlGetFirstChildByTagName(nameNode,"affiliation");
		  while (istexAffiliation)
		    {
		      SxmlNode *teiAffiliation;
		      SxmlAppendChild(teiAuthor, teiAffiliation=SxmlElementCreate("affiliation"));
		      SxmlAppendChild(teiAffiliation, SxmlLeafCreate("mods:affiliation", SxmlLeafText(istexAffiliation)));
		      istexAffiliation=SxmlGetNextSiblingByTagName( istexAffiliation,"affiliation");
		    }
		  SxmlAppendChild(Tei_analytic, SxmlClone(teiAuthor));
		} /* end if given && family */
	    } /* end if aut */
	} /* end if personal */
      nameNode=SxmlGetNextSiblingByTagName(nameNode, "name");
    } /* end while nameNode */
}

int main()
{
  bufSort=BufferCreate(10,10);
  bufUniq=BufferCreate(10,10);

  while ((docInput=SxmlInputNextDocumentElement()))
    {
      SxmlNode *docu;
      /* SxmlNode *metaNode; */
      corpusRecord=SxmlElementCreate("record");
      SxmlAppendChild(corpusRecord, teiRoot=TeiHandlerNew());
      SxmlSetAttribute(teiRoot, "xmlns:mods", "http://www.loc.gov/mods/v3");
      
      docu=SxmlClone(docInput);
      SxmlSetAttribute(docu, "xmlns:istex", "hhtp://api.istex.fr");
      SxmlSetAttribute(docu, "xmlns:json", "http://ticri.univ-lorraine.fr");
      SxmlSetAttribute(docu, "xmlns:wicri", "http://ticri.univ-lorraine.fr");
      SxmlAppendChild(corpusRecord,docu);
      /*
      if ((metaNode=SxmlGetFirstChildByTagName(docInput,"metadata")))
	{
	  if (!(modsRoot=SxmlGetFirstChildByTagName(metaNode,"mods")))continue;
	}
      else continue;
      */
      modsRoot=IstexRecordGetModsElement(docInput);
      if(!modsRoot) continue;
      /* istexRelatedItem = SxmlGetFirstChildByTagName(modsRoot,"relatedItem"); */
      istexRelatedItem = SxmlGetFirstChildTagAtt(modsRoot,"relatedItem","type","host"); 

      computeId();
      computeTitle();
      computeDate();
      computeDoi();
      computeUrl();
      computeISSN();
      computeAbstract();
      computeRelated();
      computeTopics();
      setTeiAuthors();

      SxmlPrint(corpusRecord);
      putchar('\n');
      SxmlFree(corpusRecord);
    }
  exit (EXIT_SUCCESS);
}
