/*   -*- coding: utf-8 -*-  */

/**********************************************************************
*
*  module   : Istex
*  commande  : IstexToTei
*  fichier  : IstexToTei.c
*  Auteur   : Jacques DUCLOY
*  Date     : 2015
*
***********************************************************************/

#include <stdio.h>     /* for printf */
#include <stdlib.h>    /* for exit */
#include <ctype.h>
#include <getopt.h>
#include "TeiHandler.h"
#include "Buffer.h"
#include "Utf8Converter.h"
#include "IstexRecord.h"

SxmlNode *docInput;
SxmlNode *corpusRecord;
SxmlNode *teiRoot;
SxmlNode *modsRoot;

SxmlNode *IstexTei_analytic;
SxmlNode *IstexTei_monogr;
SxmlNode *IstexTei_series;

SxmlNode *istexRelatedItem;

Buffer *bufSort;
Buffer *bufUniq;
Buffer *bufMessErr;

void computeTitle()
{
  SxmlNode *title;
  SxmlNode *titleInfo;
  char *lang;
  if ((titleInfo=SxmlGetFirstChildByTagName(modsRoot,"titleInfo")))
    {
      if ((title=SxmlGetFirstChildByTagName(titleInfo,"title")))
	{
	  SxmlNode *titleField;
	  titleField=SxmlClone(title);
	  SxmlAppendChild(Tei_titleStmt,titleField);
	  lang=SxmlGetAttribute(titleInfo,"lang");
	  if (lang)                             /* to be improved */
	    {
	      if (strncmp(lang, "en",2)==0)SxmlSetAttribute(titleField, "xml:lang", "en");
	      else SxmlSetAttribute(titleField, "xml:lang", lang);
	    }
	}
    }
  if (IstexTei_analytic)
    {
      SxmlNode *titleAnalytic;
      if ((titleAnalytic=SxmlGetFirstChildByTagName(IstexTei_analytic, "title")))
	{
	  SxmlAppendChild(Tei_analytic, SxmlClone(titleAnalytic));
	}
    }
}

void computeDate()
{
  SxmlNode *originInfo;
  SxmlNode *originInfoRelated;
  SxmlNode *relatedItem;
  SxmlNode *copyrightDate;
  SxmlNode *targetDate;
  char *dateStr;
  SxmlNode *teiDate;

  targetDate=NULL;
  originInfo=SxmlGetFirstChildByTagName(modsRoot,"originInfo");

  if (originInfo)
    {
      copyrightDate=SxmlGetFirstChildByTagName(originInfo,"copyrightDate");
      if (copyrightDate) targetDate=copyrightDate;
    }

  if (!targetDate)
    {
      if ((relatedItem=SxmlGetFirstChildByTagName(modsRoot,"relatedItem"))
	  && (originInfoRelated=SxmlGetFirstChildByTagName(relatedItem,"originInfo")))
	{
	  copyrightDate=SxmlGetFirstChildByTagName(originInfo,"copyrightDateRelated");
	  targetDate=copyrightDate;
	}
    }

  if ((!targetDate)&&(originInfo))
    {
      targetDate=SxmlGetFirstChildByTagName(originInfo,"dateCreated");
      if (!targetDate) targetDate=SxmlGetFirstChildByTagName(originInfo,"dateIssued");
    }
  if ((!targetDate)&&(originInfoRelated))
    {
      targetDate=SxmlGetFirstChildByTagName(originInfoRelated,"dateCreated");
    }
  if (targetDate)
    {
      dateStr=SxmlLeafText(targetDate);
      /*
      SxmlAppendChild(Tei_publicationStmt, teiDate=SxmlLeafCreate("date", dateStr));
      SxmlSetAttribute(teiDate,"when", dateStr);
      */
      teiDate=TeiSetDatePublication(dateStr);
    }
  else 
    {
      SxmlAppendChild(Tei_publicationStmt, teiDate=SxmlLeafCreate("date", "????"));
      SxmlSetAttribute(teiDate,"when", "????");
    }
}

char* computeId()
{
  SxmlNode *recordInfo;
  SxmlNode *recordIdentifierNode;
  SxmlNode *istexNode;
  char *recordIdentifier;
  istexNode=SxmlGetFirstChildTagAtt (modsRoot, "identifier", "type", "istex"); 
  if (istexNode)
    {
      recordIdentifier=SxmlLeafText(istexNode);
      TeiSetIdnoRbid("ISTEX", recordIdentifier);
      return recordIdentifier;
    }
  recordInfo=SxmlGetFirstChildByTagName(modsRoot,"recordInfo");
  recordIdentifier=NULL;
  if (recordInfo)
    {
      if((recordIdentifierNode=SxmlGetFirstChildByTagName(recordInfo,"recordIdentifier")))
	{
	  recordIdentifier=SxmlLeafText(recordIdentifierNode);
	  TeiSetIdnoRbid("ISTEX", recordIdentifier);
	}
    }
  if (!recordIdentifier)
    {
      SxmlNode *idNode;
      idNode=SxmlGetFirstChildByTagName(docInput, "id");
      if (idNode)
	{
	  recordIdentifier=SxmlLeafText(idNode);
	  TeiSetIdnoRbid("ISTEX", recordIdentifier);
	}
    }
  return recordIdentifier;
}

void computeDoi()
{
  SxmlNode *doiNode;
  doiNode=SxmlGetFirstChildTagAtt (modsRoot, "identifier", "type", "DOI"); 
  if (!doiNode)doiNode=SxmlGetFirstChildTagAtt (modsRoot, "identifier", "type", "doi"); 
  if (doiNode)  TeiAppendIdno("doi", SxmlLeafText(doiNode));
}

void computeAbstract()
{
  SxmlNode *absNode;
  absNode=SxmlGetFirstChildByTagName(modsRoot,"abstract");
  if (!absNode) return;
  while (absNode)
    {
      TeiHandlerStoreAbstract(absNode, SxmlGetAttribute(absNode, "lang"));
      absNode=SxmlGetNextSiblingByTagName(absNode,"abstract");
    }
}

void computeRelated()
{
  SxmlNode *doiNode;
  if (!istexRelatedItem)return;
  doiNode=SxmlGetFirstChildTagAtt (istexRelatedItem, "identifier", "type", "doi"); 
  if (!doiNode)return;
  TeiHandlerSetSeriesDoi(SxmlLeafText(doiNode));
}

void computeISSN()
{
  SxmlNode *issnNode;
  if (!istexRelatedItem)return;
  issnNode=SxmlGetFirstChildTagAtt (istexRelatedItem, "identifier", "type", "ISSN"); 
  while (issnNode)
    {
      char *issnVal;
      issnVal=SxmlLeafText(issnNode);
      if (strncmp(issnVal, "Electronic:",11)==0)TeiHandlerSetEIssn(issnVal+12);
      else TeiHandlerSetIssn(issnVal);
      issnNode=SxmlGetNextSiblingTagAtt (issnNode, "identifier", "type", "ISSN"); 
    }
}

void computeUrl()
{
  SxmlNode *fulltextNode;
  SxmlNode *jsonItem;
  fulltextNode=SxmlGetFirstChildByTagName(docInput,"fulltext");
  if (!fulltextNode) return;
  SxmlReset(fulltextNode);
  while ((jsonItem=SxmlNextNode(fulltextNode)))
    {
      SxmlNode *extension;
      SxmlNode *uri;
      extension=SxmlGetFirstChildByTagName(jsonItem, "extension");
      if (!extension)continue;
      uri=SxmlGetFirstChildByTagName(jsonItem, "uri");
      if (!uri)continue;
      if (SxmlIsLeaf(extension)&&(strcmp(SxmlLeafText(extension),"pdf")==0))
	{
	  TeiAppendIdno("url", SxmlLeafText(uri));
	}
      
    }
  
}

void computeTopics()
{
    SxmlNode *subjectNode;
    SxmlNode *topicNode;
    char *lang;
    subjectNode=SxmlGetFirstChildByTagName(modsRoot,"subject");
    while (subjectNode)
      {
	lang=SxmlGetAttribute(subjectNode, "lang");
	if ((lang)&&(strcmp(lang, "eng")==0))
	  {
	    topicNode=SxmlGetFirstChildByTagName(subjectNode,"topic");
	    while (topicNode)
	      {
		static Buffer *bufKw=NULL;
		char *newKw;
		if (!bufKw)bufKw=NewBuffer();
		BufferStrcpy (bufKw, newKw=SxmlLeafText(topicNode));
		if (islower(newKw[0]))newKw[0]=toupper(newKw[0]);
		TeiKeywordsPutTerm("KwdEn", NULL, NULL, "en", newKw);
		topicNode=SxmlGetNextSiblingByTagName(topicNode, "topic");
	      }
	  }
	subjectNode=SxmlGetNextSiblingByTagName(subjectNode, "subject");
      }
}

int isAuthor(SxmlNode *nameNode)   /* return an integer between 0 and 100 */
{
  SxmlNode *role;
  SxmlNode *roleTerm;
  role=SxmlGetFirstChildByTagName(nameNode,"role");
  if (!role) return 90;    /* author implicit ????? */
  roleTerm=SxmlGetFirstChildByTagName(role,"roleTerm");
  if (strcmp(SxmlLeafText(roleTerm),"author")==0) return 100;
  if (strcmp(SxmlLeafText(roleTerm),"editor")==0) return 0;
  return 80;
}

void setTeiAuthors()
{
  SxmlNode *nameNode;
  SxmlNode *teiAuthor;
  SxmlNode *teiName;
  static Buffer *bufGiven=NULL;
  static Buffer *bufName=NULL;
  static Buffer *bufFamily=NULL;
  if (!bufGiven)bufGiven=NewBuffer();
  if (!bufFamily)bufFamily=NewBuffer();
  if (!bufName)bufName=NewBuffer();
  nameNode=SxmlGetFirstChildByTagName(modsRoot,"name");
  while (nameNode)
    {
      if (SxmlHasAttribute(nameNode, "type", "personal"))
	{
	  SxmlNode *namePart;
	  SxmlNode *istexAffiliation;
	  int isAut;
	  char *given;
	  char *family;
	  given=NULL;
	  family=NULL;
	  BufferReset(bufGiven);
	  isAut=isAuthor(nameNode);
	  if (isAut>0)
	    {
	      int oneGiven;
	      oneGiven=0;
	      namePart=SxmlGetFirstChildByTagName(nameNode,"namePart");
	      while (namePart)
		{
		  if (SxmlHasAttribute(namePart,"type","given"))
		    {
		      given=SxmlLeafText(namePart);
		      if (given)
			{
			  if (oneGiven)
			    {
			      BufferStrcat(bufGiven," ");
			      BufferStrcat(bufGiven,given);
			    }
			  else
			    {
			      oneGiven=1;
			      BufferStrcpy(bufGiven,given);
			    }
			  while (BufferTailCmp(bufGiven," ")==0) BufferTailCut(bufGiven,1);
			}
		    }
		  if (SxmlHasAttribute(namePart,"type","family")) family=SxmlLeafText(namePart);
		  namePart=SxmlGetNextSiblingByTagName(namePart,"namePart");
		}
	      if (family)
		{
		  teiAuthor=SxmlElementCreate("author");
		  teiName=TeiAuthorNameFromFirstLastNamesStr(BufferString(bufGiven), family);
		  if(isAut<100)
		    {
		      static char *bNum=NULL;
		      if (!bNum) bNum=malloc(4);
		      sprintf(bNum,"%d%c", isAut,'%');
		      SxmlSetAttribute(teiAuthor, "wicri:is", bNum);
		    }
		  SxmlAppendChild(Tei_titleStmt,teiAuthor);
		  SxmlAppendChild(teiAuthor, teiName);
		  istexAffiliation=SxmlGetFirstChildByTagName(nameNode,"affiliation");
		  while (istexAffiliation)
		    {
		      SxmlNode *teiAffiliation;
		      SxmlAppendChild(teiAuthor, teiAffiliation=SxmlElementCreate("affiliation"));
		      SxmlAppendChild(teiAffiliation, SxmlLeafCreate("mods:affiliation", SxmlLeafText(istexAffiliation)));
		      istexAffiliation=SxmlGetNextSiblingByTagName( istexAffiliation,"affiliation");
		    }
		  if (IstexTei_analytic)      /* To be improved */
		    {
		      SxmlAppendChild(Tei_analytic, SxmlClone(teiAuthor));
		    }
		}
	    }
	}
      nameNode=SxmlGetNextSiblingByTagName(nameNode, "name");
    }
}

SxmlNode *cloneImprint(SxmlNode *input, SxmlNode *target, SxmlNode *imprint)
{
  SxmlNode *itemInput;
  SxmlNode *targetItem;
  SxmlNode *targetImprint;
  if (!imprint) SxmlAppendChild(target, targetImprint=SxmlElementCreate("imprint"));
  else targetImprint=imprint;
  SxmlReset (input);
  while ((itemInput=SxmlNextNode(input)))
    {
      SxmlAppendChild(targetImprint, targetItem=SxmlClone(itemInput));
      if  (SxmlNodeHasName(itemInput, "date"))
	{
	  if (!(SxmlFirstChild(itemInput)))
	    {
	      char *dateWhen;
	      dateWhen=SxmlGetAttribute(itemInput, "when");
	      if (dateWhen) SxmlAppendChild(targetItem, SxmlTextCreate(dateWhen));
	    }
	}
    }
  return targetImprint;
}

void copyToSeries(SxmlNode *input, SxmlNode *target)
{
  SxmlNode *itemInput;
  SxmlNode *imprint;
  imprint = NULL;
  SxmlReset (input);
  while ((itemInput=SxmlNextNode(input)))
    {
       if (SxmlNodeHasName(itemInput, "title"))
	 {
	   if (SxmlHasAttribute(itemInput,"level","j")) SxmlAppendChild(target, SxmlClone(itemInput));
	   else if (SxmlHasAttribute(itemInput,"level","s")) SxmlAppendChild(target, SxmlClone(itemInput));
	 }
       else if  (SxmlNodeHasName(itemInput, "idno"))
	 {
	   if (SxmlHasAttribute(itemInput,"type","pISSN"))
	     {
	       SxmlNode *targetItem;
	       SxmlAppendChild(target, targetItem=SxmlLeafCreate("idno", SxmlLeafText(itemInput)));
	       SxmlSetAttribute(targetItem,"type","ISSN");
	     }
	   else if  (SxmlHasAttribute(itemInput,"type","eISSN"))
	     {
	       SxmlAppendChild(target, SxmlClone(itemInput));
	     }
	 }
       else if  (SxmlNodeHasName(itemInput, "imprint")) cloneImprint( itemInput, target, imprint);
       else if  (SxmlNodeHasName(itemInput, "biblScope"))
	 {
	   SxmlNode *child;
	   if ((child=SxmlFirstChild(itemInput))
	       && (SxmlNodeHasName(child,"date")))
	     {
	       if (!imprint) SxmlAppendChild(target, imprint=SxmlElementCreate("imprint"));
	       SxmlAppendChild(imprint, SxmlClone(child));
	     }
	 }
    }
}

void computeSeries(SxmlNode *biblStruct)
{
  if(!IstexTei_series)
    {
      if((SxmlGetFirstChildTagAtt(IstexTei_monogr, "title", "level", "j"))
	 ||SxmlGetFirstChildTagAtt(IstexTei_monogr, "title", "level", "s"))
	{
	  SxmlInsertAfter(Tei_monogr, Tei_series=SxmlElementCreate("series"));
	  copyToSeries(IstexTei_monogr, Tei_series);
	}
    }
  else
    {
      copyToSeries(IstexTei_series, Tei_series);
    }
}
int main()
{
  bufSort=BufferCreate(10,10);
  bufUniq=BufferCreate(10,10);
  bufMessErr=NewBuffer();

  while ((docInput=SxmlInputNextDocumentElement()))
    {
      SxmlNode *docu;
      
      corpusRecord=SxmlElementCreate("record");
      SxmlAppendChild(corpusRecord, teiRoot=TeiHandlerNew());
      SxmlSetAttribute(teiRoot, "xmlns:mods", "http://www.loc.gov/mods/v3");
      TeiAppendIdno("wicri:source","ISTEX");
      docu=SxmlClone(docInput);
      SxmlSetAttribute(docu, "xmlns:wicri", "https://wicri-demo.istex.fr");
      SxmlAppendChild(corpusRecord,docu);
      IstexRecordInitFromInput(docInput);
      IstexRecordGetTeiElement(docInput);
      modsRoot=IstexRecordGetModsElement(docInput);

      if (IstexRecordTeiRoot)
	{	 
	  SxmlSetAttribute(teiRoot, "wicri:istexFullTextTei", "founded");
	  if (IstexRecordTeiHeader) SxmlSetAttribute(teiRoot, "wicri:istexFullTextTei", "teiHeader");
	  if (IstexRecordTeiFileDesc) SxmlSetAttribute(teiRoot, "wicri:istexFullTextTei", "fileDesc");
	  if (IstexRecordTeiSourceDesc) 
	    {
	      SxmlNode *istexTeiBiblStruct;
	      SxmlSetAttribute(teiRoot, "wicri:istexFullTextTei", "sourceDesc");
	      istexTeiBiblStruct=SxmlGetFirstChildByTagName(IstexRecordTeiSourceDesc,"biblStruct");
	      if (istexTeiBiblStruct)
		{
		  SxmlNode *teiItem;
		  int seriesFlag;
		  int anaFlag;
		  int monogrFlag;
		  
		  seriesFlag=0;
		  anaFlag=0;
		  monogrFlag=0;
		  IstexTei_analytic=NULL;
		  IstexTei_monogr=NULL;
		  IstexTei_series=NULL;
		  while ((teiItem=SxmlNextNode(istexTeiBiblStruct)))
		    {
		      if (SxmlNodeHasName(teiItem, "analytic")) 
			{
			  anaFlag++;
			  IstexTei_analytic=teiItem;
			}
		      else if  (SxmlNodeHasName(teiItem, "monogr"))
			{
			  monogrFlag++;
			  IstexTei_monogr=teiItem;
			  SxmlAppendChild(Tei_biblStruct, Tei_monogr=SxmlElementCreate("monogr"));
			}
		      else if (SxmlNodeHasName(teiItem, "series"))  
			{
			  if(!IstexTei_series)
			    {
			      IstexTei_series=teiItem;
			      SxmlAppendChild(Tei_biblStruct, Tei_series=SxmlElementCreate("series"));
			    }
			  seriesFlag++;
			}
		      else SxmlAppendChild(Tei_biblStruct,SxmlClone(teiItem));
		    }
		  BufferStrcpy(bufMessErr, "biblStruct");
		  if (anaFlag!=1) BufferStrcat(bufMessErr, ":analytic");
		  if (monogrFlag!=1) BufferStrcat(bufMessErr, ":monogr");
		  if (seriesFlag>1) BufferStrcat(bufMessErr, ":series");
		  SxmlSetAttribute(teiRoot, "wicri:istexFullTextTei", BufferString(bufMessErr));
		  computeSeries(Tei_biblStruct);
		}
	    }
	}
      else
	{
	  SxmlSetAttribute(teiRoot, "wicri:istexFullTextTei", "no");
	}
      modsRoot=IstexRecordGetModsElement(docInput);
      if(!modsRoot) continue;
      istexRelatedItem = SxmlGetFirstChildByTagName(modsRoot,"relatedItem");

      computeId();
      computeTitle();
      computeDate();
      computeDoi();
      computeUrl();
      computeISSN();
      IstexComputeLangUsageFromTei();
      computeAbstract();
      computeRelated();
      /* computeTopics(); */
      IstexComputeKeywordsFromTei();
      IstexComputeKeywordsFromEnrichments();
      setTeiAuthors();

      SxmlPrint(corpusRecord);
      putchar('\n');
      SxmlFree(corpusRecord);
    }
  exit (EXIT_SUCCESS);
}
