/*   -*- coding: utf-8 -*-  */

/***********************************************************************
*
*        Module   : Istex
*        Fichier  : JsonTools.c
*        Auteur   : Jacques Ducloy
*        Date     : Aout 2014
*
************************************************************************/

#include "JsonTools.h"
#include "Buffer.h"
#include <stdio.h>

char *JsonToSxmlPushEscape(Buffer *b1, char *str)
{
  switch(*str)
    {
    default:
      BufferCharCat(b1, *str);
    }
  return BufferString(b1);
}

char *JsonToSxmlGetString(SxmlNode *p1, char *str)
/*  str= after dquote
    return value after qquote
*/
{
  static Buffer *bufTag=NULL;
  char *str1;
  char *dq2;

  if (!bufTag) bufTag=NewBuffer();
  BufferReset(bufTag);
  str1=str;
  while (*str1)
    {
      switch(*str1)
	{
	case '\"':
	  SxmlAppendChild(p1, SxmlTextCreate(BufferString(bufTag)));
	  return str1+1;
	case '<':
	  BufferStrcat(bufTag, "&gt;");
	  str1++;
	  continue;
	case '&':
	  BufferStrcat(bufTag, "&amp;");
	  str1++;
	  continue;
	case '\\':
	  JsonToSxmlPushEscape(bufTag, str1+1);
	  str1+=2;
	  continue;
	default:
	  BufferCharCat(bufTag,*str1);
	  str1++;
	}
    }
  return NULL;
}

char *JsonToSxmlGetValue (SxmlNode *p1, char *str)
{
  static Buffer *bufTag=NULL;
  char *str1;

  if (!bufTag) bufTag=NewBuffer();
  BufferReset(bufTag);
  BufferCharCat(bufTag,*str);
  str1=str+1;
  while (*str1)
    {
      switch(*str1)
	{
	case ' ':
	case '\n':
	  SxmlAppendChild(p1, SxmlTextCreate(BufferString(bufTag)));
	  return str1+1;
	case ',':
	case '}':
	case '{':
	  SxmlAppendChild(p1, SxmlTextCreate(BufferString(bufTag)));
	  return str1;
	case '<':
	  BufferStrcat(bufTag, "&gt;");
	  str1++;
	  continue;
	case '&':
	  BufferStrcat(bufTag, "&amp;");
	  str1++;
	  continue;
	default:
	   BufferCharCat(bufTag,*str1);
	   str1++;
	}
    }
  return NULL;
}

char *JsonToSxmlGetList (SxmlNode *p1, char *str)
{
  SxmlNode *p2;
  char *str1;
  static char *errStr=NULL;

  str1=str+1;
  while (*str1)
    {
      switch (*str1)
	{
	case ' ': break;
	case '\n':break;
	case ',': break;
	case '{':
	  p2=SxmlElementCreate("json:item");
	  SxmlAppendChild (p1, p2);
	  str1=JsonElemToSxml(p2, str1);
	  continue;
	case ']':
	  return str1+1;
	case '\"':
	  p2=SxmlElementCreate("json:string");
	  SxmlAppendChild (p1, p2);
	  str1=JsonToSxmlGetString(p2, str1+1);
	  continue;
	case 'n':
	  if (strncmp(str1,"null",4)==0)
	    {
	      p2=SxmlElementCreate("json:null");
	      SxmlElementSetEmpty(p2);
	      SxmlAppendChild(p1,p2);
	      str1+=3;
	      break;
	    }
	default:
	  if (!errStr)
	    {
	      errStr=malloc(2);
	      errStr[1]='\0';
	    }
	  errStr[0]=*str1;
	  p2=SxmlLeafCreate("json:error", errStr);
	  SxmlAppendChild(p1,p2);
	  break;
	}
      str1++;
    }
  return NULL;
}

char *JsonToSxmlGetElemFromDquote(SxmlNode *p1, char *str)
{
  SxmlNode *p2;
  char *str1;
  char *dq;
  char *ddot;
  char *dq2;
  char *curCar;
  static Buffer *bufTag=NULL;

  if (!bufTag) bufTag=NewBuffer();
  str1=str+1;
  dq=strchr(str1,'\"');
  BufferStrncpy(bufTag, str1, dq-str1);
  curCar=BufferString(bufTag);
  while(*curCar)
    {
      if (curCar[0]==' ') curCar[0]='_';
      curCar++;
    }
  SxmlAppendChild(p1,p2=SxmlElementCreate(BufferString(bufTag)));
  ddot=strchr(dq,':');
  str1=ddot+1;
  while (*str1)
    {
      switch (*str1)
	{
	case ' ':break;
	case '\"':
	  /*
	  dq2=strchr(str1+1,'\"');
	  if (!dq2)return NULL;
	  BufferStrncpy(bufTag, str1+1, dq2-str1-1);
	  SxmlAppendChild(p2, SxmlTextCreate(BufferString(bufTag)));
	  return dq2+1;
	  */
	  return JsonToSxmlGetString(p2, str1+1);
	case ',': return str1+1;
	case '[': 
	  str1=JsonToSxmlGetList(p2, str1);
	  return str1;
	  /*	case '{': return str1+1; */
	case '{':
	  str+=1;
	  return JsonElemToSxml(p2, str1);
	case '}': return str1+1;
        default: 
	  str1=JsonToSxmlGetValue (p2, str1);
	  return str1;
	}
      str1++;
    }
  return dq+1;
}


char *JsonElemToSxml(SxmlNode *p1, char *str)
{
/*
          str ->   {...
	  return after }
                 or NULL
 */
  char *str1;
  str1=str+1;
  while (*str1)
    {
      switch (*str1)
	{
	case '}': return str1+1;
	case '\"': 
	  str1=JsonToSxmlGetElemFromDquote(p1,str1);
	  if (!str1)return NULL;
	  break;
	default: str1++;
	}
    }
  return NULL;
}

SxmlNode *JsonToSxml(char *str)
{
  SxmlNode *res;
  res=SxmlElementCreate("json");
  SxmlSetAttribute(res, "xmlns:json", "http://ticri.univ-lorraine.fr/wicri-meta.fr/index.php/JSON");
  if(str[0]!='{') return NULL;
  JsonElemToSxml(res, str);
  return res;
}

char * JsonBufferFromStreamGetObject(Buffer *buf, FILE *stream)
{
  int c;
  while((c=fgetc(stream))!=EOF)
    {
      BufferCharCat(buf, c);
      if (c=='}') return BufferString(buf);
      if (c=='{') JsonBufferFromStreamGetObject(buf, stream);
    }
  return NULL;
}

char *JsonBufferFromStream(Buffer *buf, FILE *stream)
{
  int c;

  BufferReset(buf);
  while((c=fgetc(stream))!=EOF)
    {
      if (c=='{')
	{
	  BufferCharCat(buf, '{');
	  return JsonBufferFromStreamGetObject(buf, stream);
	}
    }
  return NULL;
}

