/*   -*- coding: utf-8 -*-  */

/*
           NlmPubMed2CorpusTei.c

 */

#include <stdio.h>     /* for printf */
#include <stdlib.h>    /* for exit */
#include <getopt.h>
#include <ctype.h>

#include "TeiHandler.h"
#include "Buffer.h"

#include "Explor.h"

SxmlNode *docInput;
SxmlNode *corpusRecord;
SxmlNode *teiRoot;

SxmlNode *MedlineCitation;
SxmlNode *MedlineArticle;
SxmlNode *MeshHeadingList;
SxmlNode *MedlineJournal;
SxmlNode *MedlineJournalIssue ;
SxmlNode *MedlineChemicalList;
SxmlNode *PubMedPubmedData;
/*
Buffer *bufName;
Buffer *bufSort;
Buffer *bufUniq;
*/
void pubmedComputeIdno()
{
  static Buffer *bufKey=NULL;
  SxmlNode *idno;
  if (!bufKey) bufKey=NewBuffer();

  SxmlAppendChild(Tei_publicationStmt, idno=SxmlLeafCreate("idno", SxmlRecordKeyString(SxmlStdinRecord)));
  BufferStrcpy(bufKey, "wicri:Area/");
  BufferStrcat(bufKey, ExplorStreamCode);
  BufferStrcat(bufKey, "/Corpus");
  SxmlSetAttribute(idno, "type", BufferString(bufKey));

  /*  SxmlSetAttribute(idno, "type", "corpus"); */
}

 void pubmedComputeArticleTitle()
 {
   SxmlNode *ArticleTitle;
   if((ArticleTitle=SxmlGetFirstChildByTagName(MedlineArticle,"ArticleTitle")))
     {
       SxmlNode *titleTei;
       SxmlAppendChild(Tei_titleStmt,titleTei=SxmlClone(ArticleTitle));
       SxmlSetNodeName(titleTei,strdup("title"));
       SxmlSetAttribute(titleTei,"xml:lang","en");
       if (!Tei_analytic) SxmlAppendChild(Tei_biblStruct, Tei_analytic=SxmlElementCreate("analytic"));
       SxmlAppendChild(Tei_analytic, SxmlClone(titleTei));	 
     }
 }

SxmlNode *pubmedAffiliation;
int firstAuthor;

void pubmedComputeAuthorAffiliation(SxmlNode *teiAuthor, SxmlNode *MedlineAuthor)
{
  SxmlNode *pubmedAuthorAffiliation;
  SxmlNode *teiAffiliation;
  SxmlNode *pubmedAuthorAffiliationInfo;

  pubmedAuthorAffiliation=SxmlGetFirstChildByTagName(MedlineAuthor,"Affiliation");
  if (pubmedAuthorAffiliation)
    {
      SxmlAppendChild(teiAuthor, teiAffiliation=SxmlElementCreate("affiliation"));
      SxmlAppendChild(teiAffiliation, SxmlLeafCreate("nlm:affiliation", SxmlLeafText(pubmedAuthorAffiliation)));
      return;
    }
  pubmedAuthorAffiliationInfo=SxmlGetFirstChildByTagName(MedlineAuthor,"AffiliationInfo");
  if(pubmedAuthorAffiliationInfo)
    {
      while ( pubmedAuthorAffiliationInfo)
	{
	  SxmlReset(pubmedAuthorAffiliationInfo);
	  while((pubmedAuthorAffiliation=SxmlNextNode(pubmedAuthorAffiliationInfo)))
	    {
	      if (SxmlNodeHasName(pubmedAuthorAffiliation, "Affiliation"))
		{
		  SxmlAppendChild(teiAuthor, teiAffiliation=SxmlElementCreate("affiliation"));
		  SxmlAppendChild(teiAffiliation, SxmlLeafCreate("nlm:affiliation", SxmlLeafText(pubmedAuthorAffiliation)));
		}
	    }
	  pubmedAuthorAffiliationInfo=SxmlGetNextSiblingByTagName(pubmedAuthorAffiliationInfo,"AffiliationInfo" );
	}
      return;
    }
  if(firstAuthor==1)
    {
      firstAuthor=0;
      if (pubmedAffiliation)
	{
	  SxmlAppendChild(teiAuthor, teiAffiliation=SxmlElementCreate("affiliation"));
	  SxmlAppendChild(teiAffiliation, SxmlLeafCreate("nlm:affiliation", SxmlLeafText(pubmedAffiliation)));
	}
    }
}

void pubmedComputeAuthors()
 {
   SxmlNode *MedlineAuthorList;
   SxmlNode *MedlineAuthor;

   pubmedAffiliation=SxmlGetFirstChildByTagName(MedlineArticle,"Affiliation");
   if ((MedlineAuthorList=SxmlGetFirstChildByTagName(MedlineArticle,"AuthorList")))
     {
       SxmlReset(MedlineAuthorList);
       firstAuthor=1;
       while ((MedlineAuthor=SxmlNextNode(MedlineAuthorList)))
	 {
	   SxmlNode *forename;
	   SxmlNode *lastname;
	   SxmlNode *teiName;
	   SxmlNode *teiAuthor;
	   forename=SxmlGetFirstChildByTagName(MedlineAuthor,"ForeName");
	   lastname=SxmlGetFirstChildByTagName(MedlineAuthor,"LastName");

	   /*

	   if (lastname)
	     {
	       char *lastnameStr;
	       lastnameStr=SxmlLeafText(lastname);
	       BufferStrcpy(bufSort, Utf8NameToWikiSort(lastnameStr));
	       BufferStrcpy(bufUniq, Utf8NameToWikiSort(lastnameStr));
	       if (forename)
		 {
		   char *forenameStr;
		   int forenameLen;
		   forenameStr=SxmlLeafText(forename);
		   BufferStrcpy(bufName, forenameStr);
		   forenameLen=strlen(forenameStr);        
		   if ((forenameLen>3)
		       &&(forenameStr[forenameLen-2]==' ')
                       &&(isupper(forenameStr[forenameLen-1]))) BufferStrcat(bufName, ".");
		   BufferStrcat(bufName, " ");
		   BufferStrcat(bufName, SxmlLeafText(lastname));
		   BufferStrcat(bufSort, ", ");
		   BufferStrcat(bufUniq, " ");
		   BufferStrcat(bufSort, Utf8NameToWikiSort(forenameStr));
		   BufferCatChar(bufUniq, Utf8NameToWikiSort(forenameStr)[0]);
		   BufferCatChar(bufUniq, '\0');
		 }
	       else BufferStrcpy(bufName, SxmlLeafText(lastname));
	       
	       SxmlAppendChild(teiAuthor, teiName=SxmlLeafCreate("name", BufferString(bufName)));
	       SxmlSetAttribute(teiName,  "sortKey", BufferString(bufSort));
	       SxmlSetAttribute(teiName,  "uniqKey", BufferString(bufUniq));
	     }
	   */
	   teiAuthor=SxmlElementCreate("author");
	   teiName=TeiAuthorNameFromFirstLastNames(forename, lastname);
	   if (teiName)
	     {
	       SxmlAppendChild(teiAuthor,teiName);
	       SxmlAppendChild(Tei_titleStmt,teiAuthor);
	       pubmedComputeAuthorAffiliation(teiAuthor, MedlineAuthor);
	       SxmlAppendChild(Tei_analytic,SxmlClone(teiAuthor));
	     }
	 } /* end while authors*/
     }
 }

SxmlNode *pubmedMeshIsChemical (char *term)
{
  SxmlNode * chemicalNode;
  if (!MedlineChemicalList)return NULL;
  SxmlReset (MedlineChemicalList);
  while ((chemicalNode=SxmlNextNode(MedlineChemicalList)))
    {
      char *substance;
      SxmlNode *substanceNode;
      if ((substanceNode=SxmlGetFirstChildByTagName(chemicalNode, "NameOfSubstance"))
	  && (substance=SxmlLeafText(substanceNode))
	  && (strcmp(substance, term)==0))return chemicalNode;
    }
  return NULL;
}

void pubmedComputeMesh()
{
  SxmlNode *MeshHeadingList;
  SxmlNode *meshListTerm;
  static Buffer *bufMesh=NULL;
  static Buffer *bufType=NULL;
  if (!bufMesh)bufMesh=NewBuffer();
  if (!bufType)bufType=NewBuffer();
  meshListTerm=NULL;

  if ((MeshHeadingList=SxmlGetFirstChildByTagName(MedlineCitation, "MeshHeadingList")))
    {
      SxmlNode *MeshHeading;
      SxmlNode *descriptorName;
      while ((MeshHeading=SxmlNextNode(MeshHeadingList)))
	{
	  /*
	  if(!meshListTerm)
	    {	      
	      SxmlAppendChild(Tei_textClass, meshListTerm=SxmlElementCreate("keywords"));
	      SxmlSetAttribute(meshListTerm,"scheme","MESH");
	      SxmlSetAttribute(meshListTerm,"xml:lang","en");
	    }
	  */
	  if((descriptorName=SxmlGetFirstChildByTagName(MeshHeading, "DescriptorName")))
	    {
	      SxmlNode *QualifierName;
	      char *qualifier;
	      char *descriptor;
	      char *type;
	      
	      descriptor=SxmlLeafText(descriptorName);
	      if ((type=SxmlGetAttribute(descriptorName, "Type")))
		{
		  BufferStrcpy(bufType, type);
		  BufferString(bufType)[0]=tolower(type[0]);
		  type=BufferString(bufType);
		}
	      if (pubmedMeshIsChemical (descriptor)) type="chemical";

	      if((QualifierName=SxmlGetFirstChildByTagName(MeshHeading, "QualifierName")))
		{
		  SxmlNode *qualifierNode;
		  SxmlReset (MeshHeading);
		  while ((qualifierNode=SxmlNextNode(MeshHeading)))
		    {
		      if (SxmlNodeHasName(qualifierNode,"QualifierName"))
			{
			  BufferStrcpy(bufMesh,descriptor);
			  qualifier=SxmlLeafText(qualifierNode);
			  BufferStrcat(bufMesh," (");
			  BufferStrcat(bufMesh, qualifier);
			  BufferStrcat(bufMesh,")");
			  TeiKeywordsPutTerm("MESH",  type, qualifier, "en", descriptor);
			  TeiKeywordsPutTerm("KwdEn", NULL, NULL, "en", BufferString(bufMesh));
			}
		    }
		}
	      else
		{
		  TeiKeywordsPutTerm("MESH",  type, NULL, "en", descriptor);
		  /* TeiKeywordsPutTerm("KwdEn", NULL, NULL, "en", descriptor); */
		  BufferStrcpy(bufMesh,descriptor);
		  BufferStrcat(bufMesh," (MeSH)");
		  TeiKeywordsPutTerm("KwdEn", NULL, NULL, "en", BufferString(bufMesh));
		}
	    }
	} 
    }
}

void pubmedComputeDatePub()
{
  SxmlNode *MedlinePubDate;
  SxmlNode *teiDate;
  static Buffer *bufDate=NULL;
  if (!bufDate)bufDate=NewBuffer();
  if (MedlineJournalIssue)
    {
      MedlinePubDate=SxmlGetFirstChildByTagName(MedlineJournalIssue, "PubDate");
      if (MedlinePubDate)
	{
	  SxmlNode *Year;
	  SxmlNode *MedlineDate;
	  Year=SxmlGetFirstChildByTagName(MedlinePubDate, "Year");
	  if (Year)
	    {
	      char *yearStr;
	      yearStr=SxmlLeafText(Year);
	      SxmlAppendChild(Tei_publicationStmt, teiDate=SxmlLeafCreate("date", yearStr));
	      SxmlSetAttribute(teiDate,"when",yearStr);
	      TeiSeriesSetDatePublication(yearStr);
	      return;
	    }
	  MedlineDate=SxmlGetFirstChildByTagName(MedlinePubDate, "MedlineDate");
	  if (MedlineDate)
	    {
	      char *dateStr;
	      static Buffer *bufYear=NULL;
	      if (!bufYear)bufYear=NewBuffer();
	      dateStr=SxmlLeafText(MedlineDate);
	      SxmlAppendChild(Tei_publicationStmt, teiDate=SxmlLeafCreate("date", dateStr));
	      if (strlen(dateStr)>=4)
		{
		  BufferStrncpy(bufYear,dateStr,4);
		  if ((strcmp(dateStr, "1900")>0) && (strcmp(dateStr, "2100")<0))
		    {
		      SxmlSetAttribute(teiDate,"when", BufferString(bufYear));
		      return;
		    }
		}
	      SxmlSetAttribute(teiDate,"when","????");
	      return;
	    }
	  SxmlAppendChild(Tei_publicationStmt, teiDate=SxmlElementCreate("date"));
	  SxmlAppendChild(teiDate, SxmlClone(MedlinePubDate));
	  SxmlSetAttribute(teiDate,"when","????");
	  return;
	}
    }
  SxmlAppendChild(Tei_publicationStmt, teiDate=SxmlLeafCreate("date", "????"));
  SxmlSetAttribute(teiDate,"when", "????");
  return;
}

void pubmedComputeAbstractText(SxmlNode *absNode, char *lang)
{
   SxmlNode *absTextNode;
   absTextNode=SxmlGetFirstChildByTagName(absNode,"AbstractText");
   while (absTextNode)
     {
       char *label;
       label=SxmlGetAttribute (absTextNode, "Label");
       if (label)
	 {
	   SxmlNode *divNode;
	   SxmlNode *pNode1;
	   SxmlNode *pNode2;
	   SxmlNode *bNode;
	   divNode=SxmlElementCreate("div");
	   bNode=SxmlLeafCreate("b", label);
	   pNode1=SxmlElementCreate("p");
	   SxmlAppendChild(pNode1, bNode);
	   SxmlAppendChild(divNode, pNode1);
	   pNode2=SxmlLeafCreate("p", SxmlLeafText(absTextNode));
	   SxmlAppendChild(divNode, pNode2);
	   TeiHandlerStoreAbstract(divNode, lang);
	   SxmlFree(divNode);
	 }
       else TeiHandlerStoreAbstract(absTextNode, lang);
       absTextNode=SxmlGetNextSiblingByTagName(absTextNode,"AbstractText");
     }
}

void pubmedComputeAbstract()
{
  SxmlNode *absNode;

  if ((absNode=SxmlGetFirstChildByTagName(MedlineArticle, "Abstract")))
    {
      pubmedComputeAbstractText(absNode, "en");
    }
  if ((absNode=SxmlGetFirstChildByTagName(MedlineCitation, "OtherAbstract")))
    {
      char *langPubMed;
      langPubMed=SxmlGetAttribute(absNode, "Language");
      if (strcmp (langPubMed, "fre")==0) pubmedComputeAbstractText(absNode, "fr");
    }
}

void pubmedComputeArticleId()
{
  SxmlNode *PubMedArticleIdList;
  SxmlNode *PubMedArticleId;
  if (!PubMedPubmedData) return;
  PubMedArticleIdList=SxmlGetFirstChildByTagName(PubMedPubmedData, "ArticleIdList");
  if (!PubMedArticleIdList)return;
  SxmlReset(PubMedArticleIdList);
  while ((PubMedArticleId=SxmlNextNode(PubMedArticleIdList)))
    {
      if (SxmlNodeHasName(PubMedArticleId, "ArticleId"))
	{
	  if (SxmlHasAttribute(PubMedArticleId, "IdType", "doi")) 
	    {
	      TeiAppendIdno("doi", SxmlLeafText(PubMedArticleId));
	      continue;
	    }
	  if (SxmlHasAttribute(PubMedArticleId, "IdType", "pmc")) 
	    {
	      TeiAppendIdno("pmc", SxmlLeafText(PubMedArticleId));
	      continue;
	    }
	  if (SxmlHasAttribute(PubMedArticleId, "IdType", "pubmed"))
	    {
	      TeiSetIdnoRbid("pubmed", SxmlLeafText(PubMedArticleId));
	      TeiAppendIdno("pmid", SxmlLeafText(PubMedArticleId));
	    }
	}
    }
}

void pubmedComputeJournal()
{
  SxmlNode *titleJournal;
  SxmlNode *teiTitle;
  SxmlNode *issnNode;
  if (!MedlineJournal)return;
  if (!Tei_series)SxmlAppendChild(Tei_biblStruct, Tei_series=SxmlElementCreate("series"));
  if ((titleJournal=SxmlGetFirstChildByTagName(MedlineJournal, "Title")))
    {
      SxmlAppendChild(Tei_series, teiTitle=SxmlLeafCreate("title", SxmlLeafText(titleJournal)));
      SxmlSetAttribute(teiTitle, "level", "j");
    }
  issnNode=SxmlGetFirstChildByTagName(MedlineJournal, "ISSN");
  while(issnNode)
    {
      SxmlNode *idno;
      SxmlAppendChild(Tei_series, idno=SxmlLeafCreate("idno", SxmlLeafText(issnNode)));
      if (SxmlHasAttribute(issnNode, "IssnType","Print")) SxmlSetAttribute(idno, "type", "ISSN");
      else if  (SxmlHasAttribute(issnNode, "IssnType","Electronic")) SxmlSetAttribute(idno, "type", "eISSN");
      issnNode=SxmlGetNextSiblingByTagName(issnNode,"ISSN");

      
    }
}

int main(int argc, char **argv)
{
  int  cOption;
  char *pathParamFile;
  /*                                                         parameters are deprecated */
   while((cOption=getopt(argc,argv,"s:t:"))!=EOF)
     {switch (cOption)
	 {
	 case 's':
	   ExplorStreamCode=optarg;
	   break;
	 case 't':
	   pathParamFile=optarg;
	   break;
	 }
     }

   /*
  bufName=BufferCreate(10,10);
  bufSort=BufferCreate(10,10);
  bufUniq=BufferCreate(10,10);
   */
  /* ExplorParamInit(pathParamFile); */  

  while ((docInput=SxmlInputNextDocumentElement()))
    {
      if(SxmlNodeHasName(docInput,"PubmedArticle"))
	{
	  SxmlNode *docu;

	  corpusRecord=SxmlElementCreate("record");

	  SxmlAppendChild(corpusRecord, teiRoot=TeiHandlerNew());
	  SxmlSetAttribute(teiRoot, "xmlns:nlm", "http://www.nlm.org");

	  docu=SxmlClone(docInput);
	  SxmlSetNodeName(docu,strdup("pubmed")); /* strdup pour SxmlFree() */
	  SxmlAppendChild(corpusRecord,docu);

	  /* pubmedComputeIdno(); */

          if ((MedlineCitation=SxmlGetFirstChildByTagName(docInput, "MedlineCitation"))
              &&(MedlineArticle=SxmlGetFirstChildByTagName(MedlineCitation, "Article")))
            {
	      MedlineJournal=SxmlGetFirstChildByTagName(MedlineArticle, "Journal");
	      MedlineChemicalList=SxmlGetFirstChildByTagName(MedlineCitation, "ChemicalList");
	      PubMedPubmedData=SxmlGetFirstChildByTagName(docInput,"PubmedData");
	      if (MedlineJournal)MedlineJournalIssue=SxmlGetFirstChildByTagName(MedlineJournal,"JournalIssue");
	      else MedlineJournalIssue=NULL;
	      TeiAppendIdno("wicri:source","PubMed");
	      pubmedComputeArticleTitle();
	      pubmedComputeJournal();
	      pubmedComputeDatePub();
	      pubmedComputeAuthors();
	      pubmedComputeMesh();
	      pubmedComputeAbstract();
              pubmedComputeArticleId();
	    }

	  SxmlPrint(corpusRecord);
	  putchar('\n');
	  SxmlFree(corpusRecord);
	}
    }
   exit(EXIT_SUCCESS);
}
