.TH RegExp 3 "27 Jan 2004"
.SH NAME
RegExp, RegExpFind \- set of pattern matching functions based on 
Unix Extended Regular Expressions.
.SH SYNOPSIS
.PP
.nf
\#include "RegExp.h"
.fi
.PP
.nf
.ft B
RegExp\(** RegExpCreate(reStr)
	char \(**reStr;
.ft
.fi
.PP
.nf
.ft B
char\(** RegExpFind(re, str)
	RegExp \(**re;
	char \(** str;
.ft
.fi
.PP
.nf
.ft B
char\(** RegExpExtract(re, str)
	RegExp \(**re;
	char \(** str;
.ft
.fi
.PP
.nf
.ft B
char\(** RegExpBefore(re,str)
	RegExp \(**re;
	char \(** str;
.ft
.fi
.PP
.nf
.ft B
char\(** RegExpAfter(re, str)
	RegExp \(**re;
	char \(** str;
.ft
.fi
.PP
.nf
.ft B
void RegExpFree(oldRegExp)
     RegExp *oldRegExp;
.PP
.nf
.ft B
int RegExpExec(RegExp *regExp, char *strInput)
.PP
.SS Deprecated functions
.ft B
void RegExpIf_Ba 
.RB ( re , 
.BR str , 
.BR BeforeStr ,
.BR AfterPtr ,
.RI { blocIf } 
\, 
.RI { blocElse }
)
.br
.fi
.PP
.ft B
void RegExpIf_Bt 
.RB ( re , 
.BR str , 
.BR BeforeStr ,
.BR TextPtr ,
.RI { blockIf } 
\, 
.RI { blockElse }
)
.fi
.PP
.ft B
void RegExpIf_BTa 
.RB ( re , 
.BR str , 
.BR BeforeStr ,
.BR TextStr ,
.BR afterPtr ,
.RI { blockIf } 
\, 
.RI { blockElse }
)
.fi
.PP
.ft B
void RegExpIf_Ta 
.RB ( re , 
.BR str , 
.BR TextStr ,
.BR AfterPtr ,
.RI { blockIf } 
\, 
.RI { blockElse }
)
.PP
.ft B
.B	RegExp \(**re;
.br
.B	char \(** str;
.br
.B	block
.RI { blockIf }
.br
.B	block
.RI { blockElse }
.br
.I	char \(**BeforeStr
.br
.I	char \(**TextStr
.br
.I	char \(**TextPtr
.br
.I	char \(**AfterPtr
.PP
.ft B
void RegExpDoBySep
.RB ( re , 
.BR str , 
.BR token ,
.RI { blockDo })
.br
.B	RegExp \(**re;
.br
.B	char \(** str;
.br
.I      char \(**token
.B	block
.I      blockDo
.PP
.ft B
void RegExpStrDoBySep
.RB ( reStr , 
.BR str , 
.BR token ,
.RI { blockDo })
.br
.B	char \(**reStr;
.br
.B	char \(** str;
.br
.I      char \(**token
.B	block
.I      blockDo
.PP
.SH DESCRIPTION
.PP
.B RegExp
is a set a functions using standard Unix 
.I regex.h
include file,
with (we hope that !) a better user interface. They handle Advanced Regular Expressions (see
.BR regex (5)).
.TP
.B RegExpCreate
compiles 
.I reStr
(and optimises memory allocation of compiled code).
.TP
.B RegExpFree
frees the 
.IR oldRegExp ,
don't forget to call it!
.PP
If 
.I re
matches
.I str,
.B RegExpFind
returns a pointer on the beginning of the matched substring,
.B RegExpAfter
returns a pointer on the first character after the matched substring,
.B RegExpExtract
returns a pointer on a copy of the matched substring, 
.B RegExpBefore
returns a pointer on a copy of substring which is before the matched substring.
.PP
Pay attention, user must manage (free) the strings which are returned by 
.B RegExpExtract
and
.B RegExpBefore.
Each function returns NULL if 
.I re
does not match
.I str.
.TP
.B RegExpExec
executes the search of 
.I regExp
in 
.IR strInput .
If it is in, sets
.I RegExpLoc1 
to the beginning of the matching substring.
.TP
.B RegExpLoc2
is an extern variable which contains a pointer on the character following the
matched string, after a
.BR RegExpExtract ,
or a 
.B RegExpExec
call.
.SS Deprecated macros
.PP
.BR RegExpIf_Ba ,
.BR RegExpIf_Bt
and 
.B RegExpIf_Ta
are macros procedures.
These macros execute pattern matching and process
.I blocIf
in case of success, and
.I blocElse
if not.
According to profiles
.BR Ba ,
.BR Bt
or
.BR Ta
strings or pointers.
.PP
Due to allocation reasons,
.I blocIf
and 
.I blocElse
must not contain any break (break, return, continue) statements.
.PP
In writing profiles, capital letters are used for new strings extracted from
initial strings, lower cases letters are dedicated to 
pointers on initial string.
All variables:
.IR BeforeStr ,
.IR TextStr ,
.IR TextPtr
and
.I AfterPtr
are declared and build before beginning of 
.I blocIf
and (if required for strings) freed at the end of
.IR blocIf .
Variables are not defined outside RegExp if macros.
Pointers get correct value outside if macro.
Strings must be copied if they have to be used outside.
.PP
.B RegExpDoBySep
analyses string
.I str
considering
.I re
as a separator pattern and executes
.I blockDo
for each string
.I token
extracted from
.IR str .
.SH WARNING
.PP
RegExp macros must be used with an extreme care as soon as they
are used in a large repetitive context.
.PP
RegExp If macros have been designed with Sgml string handling (strings
are copied into Sgml Nodes), in this case allocation and free operations
are quite transparent. 
It is not the same in all applications.
.PP
For instance, avoid programming like this:
.nf
	RegExpIf_Ba (...,
		{
		 ...
		return ;   /* RegExpIf local strings are not freed */
		},{...});
.fi
.PP
do like this:
.nf
	RegExpIf_Ba (...,
		{
		 ...
		},{...});
	return ;   /* RegExpIf local strings are freed */
.fi
.SH EXAMPLES
.PP
Looking for the first number in a string and printing it:
.PP
.nf
	RegExp *numberRe;
	char   *numberStr;

	numberRe  = RegExpCreate("[0-9]*[.]?[0-9][0-9]*");
	numberStr = RegExpExtract(numberRe,
	                          "pi is equal to 3.14 I presume...");
	if(numberStr) {
	  printf("%s\n",numberStr);
	  free(numberStr);
	} else 
	  printf("no number in this sentence\n");
	
	RegExpFree(numberRe);

.PP
The same thing, using the deprecated macro RegExpIf_Ta.
.PP
.nf
	RegExp * numberRe;

	numberRe = RegExpCreate("[0-9]*[.]?[0-9][0-9]*");
	RegExpIf_Ta (numberRe, "pi is equal to 3.14 I presume...",
	             number, after,
		{
		  printf("%s", number);
		},
		{
		  printf("no number in this sentence");
		}
	)

.SH WARNINGS
.PP
The regular expressions available here are no more 
.I Basic
Regular Expressions, as seen in
.BR regexp (5),
so they do include 
.IR + ,
.IR ? ,
or 
.I | 
special characters, since v0.4.00035 (29 Jan 2004).

.PP
.I BeforeStr
and
.I TextStr
are not available outside
.I blocIf
\.
.PP
These functions used standard regexp.h regular expressions and their limitations.
(all lex meta-character and "+" for instance were not available)
.PP
Handling of \\, \\t and \\n in c string for regular expression is complex :
.PP
Single \\ to backquote regexp command must be duplicated, sample:
.PP
	[0-9]*\\\\.[0-9]*
.PP
tabular mark is considered as standard character
and does not be backquoted, for instance:
.PP
	[ \\t]*
.PP 
I dont know how to handle \\n !!!!!!!!!!!!!
.SH AUTHORS
INIST-CNRS: Jacques DUCLOY, Francois PARMENTIER

<DilibMaster@inist.fr>
.SH "SEE_ALSO"
.BR grep (1),
.BR regcomp (3),
.BR regex (5),
.BR RegExpIntro (1).
