/****************************************************************************
*
*      Projet  : Www
*      Module  : Ssi
*      Fichier : SsiTools.c
*      Auteur  : J. Ducloy
*
*      Date    : Janvier 99
*
****************************************************************************
*
* Copyright (c) 1999 - LORIA
* 
****************************************************************************/
#include "SsiParser.h"
#include "SgmlPath.h"
#include "Except.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>


SsiParser *SsiParserCreate()
{
  SsiParser *sp1;
  char *timeSsi;
  time_t ttt;
  if(!(sp1=(SsiParser *)malloc(sizeof(SsiParser))))
    {
      ExceptSetError("SsiParser", "MA", "Memory allocation failed in SsiParserCreate","","",2);
      return NULL;
    }
  sp1->outputFile=NULL;
  sp1->dict=NULL;
  sp1->memory=StrSearchTableCreate(100,100);
  sp1->intBuf=BufferCreate(10,10);
  sp1->keyBuf=BufferCreate(10,10);
  sp1->bufOutput=BufferCreate(10,10);
  sp1->prog=NULL;
  sp1->context=NULL;
  sp1->modeBuffer=0;

  ttt=time (NULL);
  timeSsi= strdup(ctime(&ttt));
  StrSearchAdd(sp1->memory, "_TIME", SgmlDataCreate(timeSsi));
  
  return sp1;
}

void SsiParserFree(sp1)
     SsiParser *sp1;
{
  BufferFree(sp1->intBuf);
  StrSearchTableFreeStr(sp1->dict);
  StrSearchTableFree(sp1->memory);
  BufferFree(sp1->bufOutput);
  free(sp1);
}
void SsiParserPrint(sp1,s1)
    SsiParser *sp1;
     char *s1;
{
  if (s1)
    {
      if(sp1->modeBuffer)BufferStrcat(sp1->bufOutput,s1);
      else fprintf(sp1->outputFile,"%s",s1);
    }
}
SsiParser *SsiParserSetContext(sp1, c1)
     SsiParser *sp1;
     char *c1;
{
  if(sp1->context)
    {
      free(sp1->context);
      sp1->context=NULL;
    }
  if (!c1)return sp1;
  sp1->context=strdup(c1);
  if(sp1->context)return NULL;
  else return sp1;
}

void SsiParserCatMenKey(sp1, b1, k1)
SsiParser *sp1;
char *k1;
Buffer *b1;
{
  char *posPar;
  if (!k1)return;
  if (!k1[0])return;
  if (k1[0]=='(')
    {
      char *memKey;
  
      SgmlNode *v1;
      if ((posPar=strchr(k1, ')')))
	{
	  memKey=malloc(posPar-k1);
	  memKey[0]='\0';
	  strncat (memKey, k1+1, posPar-k1-1);
	}
      else memKey=k1+1;
      if (memKey[0]=='$')
	{
	  char *val;
	  if ((val=getenv(memKey+1)))BufferStrcat(b1,val);
	}
      else 
	{
	  if ((v1=(SgmlNode *)StrSearch(sp1->memory, memKey)))
	    {
	      char *val;
	      val = SgmlToString(v1);
	      BufferStrcat(b1,val);
	      free(val);
	    }
	}
      if (posPar)
	{
	  free (memKey);
	  SsiParserCatMenKey(sp1, b1, posPar+1);
	}
      return;
    }
  if ((posPar=strchr(k1,'(')))
    {
      BufferStrncat(b1, k1, posPar-k1);
      SsiParserCatMenKey(sp1, b1, posPar);
    }
  else
    {
      BufferStrcat(b1, k1);
      return;
    }
}

SsiParser *SsiParserSetProg(sp1, c1)
     SsiParser *sp1;
     char *c1;
{
  if(sp1->prog)
    {
      free(sp1->prog);
      sp1->prog=NULL;
    }
  if (!c1)return sp1;
  sp1->prog=strdup(c1);
  if(sp1->prog)return NULL;
  else return sp1;
}

SsiParser *SsiParserSetDict(sp1, f1)
     SsiParser *sp1;
     char *f1;
{
  if (!f1)return (NULL);
  if (sp1->dict)
    {
      StrSearchAddTable( sp1->dict, f1);
    }
  else
    {
      sp1->dict=StrSearchGetTable(f1,100);
    }
  return sp1;
}

SsiParser * SsiParserSetOutput(sp1,f1)
SsiParser *sp1;
FILE *f1;
{
  if (sp1->outputFile)fclose(sp1->outputFile);
  sp1->outputFile=f1;
  return sp1;
}

void SsiParserInter(sp1,ss1)
SsiParser *sp1;
SgmlNode  *ss1;
{
  char *attVal;

  if(SgmlHasTag(ss1,"echo"))
    {
      if((attVal=SgmlGetAttValById(ss1,"var")))
	{
	  char *v;
	  if((v=getenv(attVal)))SsiParserPrint(sp1,v);
	  return;
	}
      else if ((attVal=SgmlGetAttValById(ss1,"key")))
	{
	  SsiParseKey(sp1,attVal);
	  return;
	}
      else if ((attVal=SgmlGetAttValById(ss1,"keyVar")))
	{
	  char *v;
	  if((v=getenv(attVal)))SsiParseKey(sp1,v);
	  return;
	}
      else if ((attVal=SgmlGetAttValById(ss1,"memory")))
	{
	  char *path;
	  if ((path=SgmlGetAttValById(ss1,"path")))
	    {
	      SgmlPathProg *prog;
	      SgmlPathIterator *iter;
	      SgmlNode *mNode;
	      prog = SgmlPathCompile(path);
	      iter = SgmlPathIteratorCreate(prog);
	      if ((mNode=(SgmlNode *)StrSearch (sp1->memory, attVal)))
		{
		  SgmlNode *res;
		  SgmlPathIteratorInit(iter,mNode);
		  res=SgmlPathNext(iter);
		  if(res)SgmlPrint(res);
		  
		}
	      
	      SgmlPathIteratorFree(iter);
	      SgmlPathProgFree(prog);
	    }
	  else
	    {
	      SgmlPrint((SgmlNode *)StrSearch (sp1->memory, attVal));
	    }
	  return;
	}

    }
  else if (SgmlHasTag(ss1,"include"))
    {
      if((attVal=SgmlGetAttValById(ss1,"file")))
	{
	  FILE *inputFile;
	  char *att1;
	  char *fileName;
	  char exceptFlag;

	  if ((att1=SgmlGetAttValById(ss1,"except")))
	    {
	      if (strcmp(att1,"ignore")==0)exceptFlag='i';
	      else if (strcmp(att1,"ignore")==0)exceptFlag='w';
	      else exceptFlag='a';  /* abort */
	    }
	  else exceptFlag='a';  /* abort */

	  if((att1=SgmlGetAttValById(ss1,"var")))
	    {
	      static Buffer *bName=NULL;
	      char *envVar;
	      if (!bName)bName=BufferCreate(10,10);
	      envVar=getenv(att1);
	      if (!envVar)
		{
		  switch (exceptFlag)
		    {
		    case 'i':return;
		    case 'a':
		      ExceptSetError("SsiTools","UF","var ", att1, " undefined",2);
		      exit (2);
		    case 'w':
		      ExceptSetError("SsiTools","UF","var ", att1, " undefined",0);
		      return;
		    }
		}
	      BufferStrcpy(bName,envVar);
              BufferStrcat(bName,"/");
	      BufferStrcat(bName,attVal);
	      fileName=BufferString(bName);
	    }
	  else fileName=attVal;
	  
	  if ((inputFile=fopen(fileName,"r")))
	    {
	      SsiParseFile(sp1, inputFile);
	      fclose(inputFile);
	      return;
	    }
	  else 
	    {
	      switch (exceptFlag)
		{
		case 'i':return;
		case 'a':
		  ExceptSetError("SsiTools","UF","File ", attVal, "does not exist or not fit",2);
		  exit(2);
		case 'w':
		  ExceptSetError("SsiTools","UF","File ", attVal, "does not exist or not fit",2);
		  return;
		}
	      return ;
	    }
	}
    }

  else if (SgmlHasTag(ss1,"setm"))
    {
      char *varName;
      if(!(varName=SgmlGetAttValById(ss1,"var")))
	{
	  ExceptSetError("SsiTools","SY"," Attribute var is mandatory for setm ", "", "",2);
	  return ;
	}
      if((attVal=SgmlGetAttValById(ss1,"file")))
	{
	  SgmlNode *doc;
	  Buffer *bufName;
	  char *posPar;
	  if ((posPar=strchr(attVal,'(')))
	    {
	      bufName=BufferCreate(10,10);
	      BufferStrncpy(bufName, attVal, posPar-attVal);
	      SsiParserCatMenKey(sp1, bufName, posPar);
	      if ((doc=SgmlFromFile(BufferString(bufName))))
		{
		  StrSearchPut(sp1->memory, varName,(char *)doc);
		  BufferFree(bufName);
		  return;
		}
	      else 
		{
		  ExceptSetError("SsiTools","UF","File ", BufferString(bufName), "does not exist or not fit",2);
		  return ;
		}
	    }
	  else if ((doc=SgmlFromFile(attVal)))
	    {
	      StrSearchPut(sp1->memory, varName,(char *)doc);
	      return;
	    }
	  else 
	    {
	      ExceptSetError("SsiTools","UF","File ", attVal, "does not exist or not fit",2);
	      return ;
	    }
	}
    }
  /*   SSI not handled by DILIB */
  BufferTailCut(sp1->intBuf,1);
  if(sp1->modeBuffer)
    {
      BufferStrcat(sp1->bufOutput,"<!--#");
      BufferStrcat(sp1->bufOutput,BufferString(sp1->intBuf)+1);
      BufferStrcat(sp1->bufOutput,"<!--#");
    }
  else printf("<!--#%s-->",BufferString(sp1->intBuf)+1);
  return;
}
SgmlNode *SsiParserGetSsi(sp1,s1)
SsiParser *sp1;
char *s1;
{
  char *pos;
  if (!s1)return NULL;
  if (!(pos=strstr(s1, "-->")))return NULL;
  BufferStrcpy(sp1->intBuf, "<");
  BufferStrncat(sp1->intBuf, s1+5, pos-s1-5);
  BufferStrcat(sp1->intBuf, ">");
  sp1->endString=pos+3;
  return(SgmlFromString(BufferString(sp1->intBuf)));
}


char *SsiSeek(sp1,s1)
SsiParser *sp1;
char *s1;
{
  char *pos;
  if (!s1)return NULL;
  if (!(pos=strchr(s1,'<')))return NULL;
  if (strncmp(pos, "<!--#",5)==0)return pos;
  else return (SsiSeek(sp1,pos+1));
}

void  SsiParseString(sp1, s1)
SsiParser *sp1;
char *s1;
{
  char *pos;

  if(!s1)return;
  if ((pos=SsiSeek(sp1,s1)))
    {
      char *s2;
      SgmlNode *ssi;
      char *nextString;
      s2=s1;
      while(s2<pos)fputc(*s2++, sp1->outputFile);
      ssi=SsiParserGetSsi(sp1,pos);
      nextString=SsiParserEndString(sp1);
      SsiParserInter(sp1, ssi);
      SsiParseString(sp1, nextString);
    }
  else
    {
      if (sp1->modeBuffer)
	BufferStrcat(sp1->bufOutput, s1);
      else fprintf(sp1->outputFile,"%s", s1);
    }
}

char* SsiSearchKey(sp1, k1)
SsiParser *sp1;
char *k1;
{
  char *ret;
  if(sp1->prog)
    {
      BufferStrcpy(sp1->keyBuf, sp1->prog);
      BufferStrcat(sp1->keyBuf, ":");
      BufferStrcat(sp1->keyBuf, k1);
      ret=StrSearch(sp1->dict, BufferString(sp1->keyBuf));
      if (ret)return ret;
    }
  return StrSearch(sp1->dict, k1);
}



char*  SsiParseKey(sp1, k1)
SsiParser *sp1;
char *k1;
{
  char *ret;
  char *posPar;
  char *k2;
  Buffer *bufName;

  bufName=NULL;

  if (!sp1->dict)return NULL;
  if (!k1)return NULL;
  if ((posPar=strchr(k1,'(')))
      {
	bufName=BufferCreate(10,10);
	BufferStrncpy(bufName, k1, posPar-k1);
	SsiParserCatMenKey(sp1, bufName, posPar);
	k2=BufferString(bufName);
      }
  else k2=k1;
  if((ret=SsiSearchKey(sp1, k2)))
    {
      SsiParseString(sp1,ret);
    }
  if (bufName)BufferFree(bufName);
  return ret;
}

char *SsiParserGetKey(sp1,key)
SsiParser *sp1;
char *key;
{
  sp1->modeBuffer=1;
  BufferReset(sp1->bufOutput);
  if(SsiParseKey(sp1, key))
    {
      sp1->modeBuffer=0;
      return BufferString(sp1->bufOutput);
    }
  return NULL;
}

void  SsiParseLine(sp1, l1)
SsiParser *sp1;
char *l1;
{
  SsiParseString(sp1, l1);
  if(sp1->modeBuffer)BufferStrcat(sp1->bufOutput,"\n");
  else fprintf(sp1->outputFile,"\n");
}

void SsiParseFile(sp1, f1)
SsiParser *sp1;
FILE *f1;
{
  Buffer *buf;
  buf=BufferCreate(100,10);
  while((BufferFgets(buf,f1)))
    {
      SsiParseLine(sp1, BufferString(buf));
    }
  BufferFree(buf);
}
