/***********************************************************************
*
*      Module  : SxmlProc
*      Fichier : SxmlProcKernel.h
*      Auteur  : J. Ducloy
*
*      Derniere mise a jour Decembre 2001
*      $Id: SxmlProcKernel.h,v 1.4 2003/06/13 16:33:01 parmentf Exp $
*
*=============================================================
*
*  Common declarations for SxmlMemory and SxmlProcessor
*
************************************************************************
* 
*     Copyright (C) 2001 CNRS INIST
*
************************************************************************/
#ifndef _SXML_PROC_KERNEL_H_
#define _SXML_PROC_KERNEL_H_

/**
   @file

   @brief Common declarations for SxmlMemory and SxmlProcessor

   @see SxmlProcessor SxmlMemory

   @since Dec 2001
   @author &copy; INIST-CNRS
   @author Jacques DUCLOY
 */

/*
   @addtogroup SxmlProc
   @{
 */

#include "SxmlNode.h"

/**
  SxmlMemory inherits from SxmlNode. It contains:
  - Table of symbols (named @c "#symbolTable", of type SxmlSearchTable)
  - Scripts area (for storing functions, name @c "#scripts")
  - a set of attributes for each interpret
    @c interpret:init

  @see SxmlSearchTable
 */
typedef SxmlNode SxmlMemory;
/** 
    Gets @a x's SymbolTable node 

    @param x @ref SxmlMemory
 */
#define SxmlMemorySymbolTable(x)   (SxmlFirstChild(x))
/** 
    Gets @a x's Scripts node
    
    @param m @ref SxmlMemory
 */
#define SxmlMemoryScripts(m)       (SxmlNextSibling(SxmlFirstChild(m)))

/**
    SxmlProcessor inherits from SxmlNode.
    
    It contains:
    - an @ref SxmlMemory
    - ...
 */
typedef SxmlNode SxmlProcessor;
/** 
    Gets @a x's @ref SxmlMemory 
    
    @param x @ref SxmlProcessor
 */
#define  SxmlProcessorMemory(x) ((SxmlNode *)SxmlNodeValue(SxmlFirstChild(x)))

#define SXML_PROCESSOR            1
#define SXML_PROCESSOR_MEMORY     2
#define SXML_PROCESSOR_FUNCTION   3
#define SXML_PROCESSOR_STEP       4

/*
   @}
 */

#endif  /* _SXML_PROC_KERNEL_H_  */

