/*   -*- coding: utf-8 -*-  */
#include <stdio.h>
#include <stdlib.h>
#include "SxmlNode.h"
#include "Buffer.h"
#include "SvgPath.h"

typedef SxmlNode SvgPathCommand;

SvgPathCommand *SvgPathCreateCommandFromString(char *strPath, char **toParse)
{
  SvgPathCommand *result;
  char *str;
  char carPath;
  char *str1;
  static char c2[2];

  str=strPath;
  c2[1]='\0';
  while (str[0]==' ')str++;
  switch ((carPath=str[0]))
    {
    case 'z':
    case 'Z':
      result=SxmlElementCreate("Z");
      *toParse=str+1;
      return result;

    case 'l':
    case 'L':
    case 'm':
    case 'M':
      c2[0]=carPath;
      str++;
      result=SxmlElementCreate(c2);
      while (str[0]==' ')str++;
      SxmlAppendChild(result, SxmlLeafDoubleCreate("x", strtod(str, &str1)));
      str=str1;
      while ((str[0]==' ')||(str[0]==','))str++;
      SxmlAppendChild(result, SxmlLeafDoubleCreate("y", strtod(str, &str1)));
      *toParse=str1;
      return result;
    case 'c':
    case 'C':
      c2[0]=carPath;
      str++;
      result=SxmlElementCreate(c2);
      while (str[0]==' ')str++;
      SxmlAppendChild(result, SxmlLeafDoubleCreate("x1", strtod(str, &str1)));
      str=str1;
      while ((str[0]==' ')||(str[0]==','))str++;
      SxmlAppendChild(result, SxmlLeafDoubleCreate("y1", strtod(str, &str1)));
      str=str1;
      while ((str[0]==' ')||(str[0]==','))str++;
      SxmlAppendChild(result, SxmlLeafDoubleCreate("x2", strtod(str, &str1)));
      str=str1;
      while ((str[0]==' ')||(str[0]==','))str++;
      SxmlAppendChild(result, SxmlLeafDoubleCreate("y2", strtod(str, &str1)));
      str=str1;
      while ((str[0]==' ')||(str[0]==','))str++;
      SxmlAppendChild(result, SxmlLeafDoubleCreate("x", strtod(str, &str1)));
      str=str1;
      while ((str[0]==' ')||(str[0]==','))str++;
      SxmlAppendChild(result, SxmlLeafDoubleCreate("y", strtod(str, &str1)));
      *toParse=str1;
      return result;
    }
  return NULL;
}

  int traceLevel;

SxmlNode *proceedPoints(SxmlNode *paramMap)
{
  SxmlNode *listPoints;
  SxmlNode *firstPoint;
  SxmlNode *nextPoint;
  double    longitude0;
  double    longitude1;
  double    longitude2;
  double    longitude9;
  double    latitude0;
  double    latitude1;
  double    latitude2;
  double    latitude9;
  double    width;
  double    height;
  double    x1;
  double    x2;
  double    Y1;
  double    Y2;

  listPoints=SxmlGetFirstChildByTagName(paramMap, "points");
  width=SxmlLeafDoubleValue(SxmlGetFirstChildByTagName(paramMap, "width"));
  height=SxmlLeafDoubleValue(SxmlGetFirstChildByTagName(paramMap, "height"));

  firstPoint=SxmlFirstChild(listPoints);
  longitude1=SxmlLeafDoubleValue(SxmlGetFirstChildByTagName(firstPoint, "longitude"));
  latitude1=SxmlLeafDoubleValue(SxmlGetFirstChildByTagName(firstPoint, "latitude"));
  x1=SxmlLeafDoubleValue(SxmlGetFirstChildByTagName(firstPoint, "x"));
  Y1=SxmlLeafDoubleValue(SxmlGetFirstChildByTagName(firstPoint, "Y"));

  nextPoint=SxmlNextSibling(firstPoint);
  longitude2=SxmlLeafDoubleValue(SxmlGetFirstChildByTagName(nextPoint, "longitude"));
  latitude2=SxmlLeafDoubleValue(SxmlGetFirstChildByTagName(nextPoint, "latitude"));
  x2=SxmlLeafDoubleValue(SxmlGetFirstChildByTagName(nextPoint, "x"));
  Y2=SxmlLeafDoubleValue(SxmlGetFirstChildByTagName(nextPoint, "Y"));

  longitude0=(longitude2*x1-longitude1*x2)/(x1-x2);
  latitude9=(latitude2*Y1-latitude1*Y2)/(Y1-Y2);
  longitude9=longitude0+width*(longitude2-longitude1)/(x2-x1);
  latitude0=latitude9+height*(latitude2-latitude1)/(Y2-Y1);

  SxmlAppendChild(paramMap, SxmlLeafDoubleCreate("left", longitude0));
  SxmlAppendChild(paramMap, SxmlLeafDoubleCreate("up", latitude0));
  SxmlAppendChild(paramMap, SxmlLeafDoubleCreate("right", longitude9));
  SxmlAppendChild(paramMap, SxmlLeafDoubleCreate("down", latitude9));

  if (traceLevel>0)  
    {
      SxmlPrint(paramMap);
      putchar('\n');
    }
  return paramMap;
}

SxmlNode *SvgSxmlPathFromSvgLeaf(SxmlNode *nodePath)
{
  SxmlNode *svgPath;
  SxmlNode *svgItem;
  char *strPath;
  char *toParse;

  svgPath=SxmlElementCreate("svgPath");
  strPath=SxmlLeafText(nodePath); 
  while ((svgItem=SvgPathCreateCommandFromString(strPath, &toParse)))
    {
      SxmlAppendChild(svgPath, svgItem);
      strPath=toParse;
    }
  return svgPath;
}

SxmlNode *GeoPathFromSvgPath(SxmlNode *svgPath, SxmlNode *paramMap)
{
  double up;
  double down;
  double right;
  double left;
  double height;
  double width;
  double scaleX;
  double scaleY;
  SxmlNode *geoPath;
  SxmlNode *svgItem;

  width=SxmlLeafDoubleValue(SxmlGetFirstChildByTagName(paramMap, "width"));
  height=SxmlLeafDoubleValue(SxmlGetFirstChildByTagName(paramMap, "height"));
  up=SxmlLeafDoubleValue(SxmlGetFirstChildByTagName(paramMap, "up"));
  down=SxmlLeafDoubleValue(SxmlGetFirstChildByTagName(paramMap, "down"));
  right=SxmlLeafDoubleValue(SxmlGetFirstChildByTagName(paramMap, "right"));
  left=SxmlLeafDoubleValue(SxmlGetFirstChildByTagName(paramMap, "left"));

  scaleX=(right-left)/width;
  scaleY=(up-down)/height;

  geoPath=SxmlElementCreate("path");

  SxmlReset(svgPath);
  while ((svgItem=SxmlNextNode(svgPath)))
    {
      SxmlNode *geoItem;
      char *commandName;
      commandName=SxmlNodeName(svgItem);
      SxmlAppendChild(geoPath, geoItem=SxmlElementCreate(commandName));
      switch(*commandName)
	{
	case 'Z':
	case 'z':
	  break;
	case 'L':
	case 'M':
	  SxmlAppendChild(geoItem, SxmlLeafDoubleCreate("x", left+SvgPathCommandX(svgItem)*scaleX));
	  SxmlAppendChild(geoItem, SxmlLeafDoubleCreate("y", up-SvgPathCommandY(svgItem)*scaleY));
	  break;
	case 'C':
	  SxmlAppendChild(geoItem, SxmlLeafDoubleCreate("x", left+SvgPathCommandX(svgItem)*scaleX));
	  SxmlAppendChild(geoItem, SxmlLeafDoubleCreate("y", up-SvgPathCommandY(svgItem)*scaleY));
	  SxmlAppendChild(geoItem, SxmlLeafDoubleCreate("x1", left+SvgPathCommandItem(svgItem, "x1")*scaleX));
	  SxmlAppendChild(geoItem, SxmlLeafDoubleCreate("y1", up-SvgPathCommandItem(svgItem, "y1")*scaleY));
	  SxmlAppendChild(geoItem, SxmlLeafDoubleCreate("x2", left+SvgPathCommandItem(svgItem, "x2")*scaleX));
	  SxmlAppendChild(geoItem, SxmlLeafDoubleCreate("y2", up-SvgPathCommandItem(svgItem, "y2")*scaleY));
	}
    }
  return geoPath;
}

void proceedPlace(SxmlNode *paramMap, SxmlNode *nodeLinkName, SxmlNode *newParam, SxmlNode *nodePath)
{
  SxmlNode *nodeName;
  SxmlNode *linkName;
  SxmlNode *placeNode;
  SxmlNode *leftNode;
  SxmlNode *geoDilib;
  SxmlNode *svgPath;
  SxmlNode *geoPath;

  if (traceLevel>0)  printf("== place ==\n"); 
  placeNode=SxmlElementCreate("place");
  nodeName=SxmlGetFirstChildByTagName(linkName=SxmlGetFirstChildByTagName(nodeLinkName,"p"), "k");
  SxmlAppendChild(placeNode, SxmlLeafCreate("placeName", SxmlLeafText(nodeName)));
  leftNode=SxmlGetFirstChildByTagName(paramMap,"left");
  if (!leftNode)leftNode=proceedPoints(paramMap);
  SxmlAppendChild(placeNode, geoDilib=SxmlElementCreate("dilib:geoPath"));
  if (traceLevel>0)  
    {
      SxmlSave(placeNode);
      putchar('\n');
    }
  svgPath=SvgSxmlPathFromSvgLeaf(nodePath);
  geoPath=GeoPathFromSvgPath(svgPath, paramMap);
  SxmlAppendChild(geoDilib, geoPath);
  if (traceLevel>0)  
    {
      SxmlSave(svgPath);
      putchar('\n');
      SxmlSave(geoPath);
      putchar('\n');

    }
  SxmlSave (placeNode);
  putchar('\n');
}

SxmlNode *proceedSvg(SxmlNode *mapParam, SxmlNode *nodeList)
{
  SxmlNode *paramList;
  SxmlNode *elem;
  if (mapParam)paramList=mapParam;
  else paramList=SxmlElementCreate("map");
  SxmlReset(nodeList);
  while ((elem=SxmlNextNode(nodeList)))
    {
      SxmlAppendChild(paramList, SxmlLeafDoubleCreate(SxmlNodeName(elem), strtod(SxmlLeafText(elem),NULL)));
    }
  if (traceLevel>0)  
    {
      SxmlSave(paramList);
      putchar('\n');
    }
  return paramList;
}

SxmlNode *proceedPoint(SxmlNode *paramList, SxmlNode *nodeList)
{
  SxmlNode *listPoints;
  SxmlNode *elem;
  SxmlNode *point;

  listPoints=SxmlGetFirstChildByTagName(paramList, "points");
  if (!listPoints)SxmlAppendChild(paramList, listPoints=SxmlElementCreate("points"));
  SxmlAppendChild(listPoints, point=SxmlElementCreate("point"));
  SxmlReset(nodeList);
  while ((elem=SxmlNextNode(nodeList)))
    {
      SxmlAppendChild(point, SxmlLeafDoubleCreate(SxmlNodeName(elem), strtod(SxmlLeafText(elem),NULL)));
    }
  if (traceLevel>0)  
    {
      SxmlSave(paramList);
      putchar('\n');
    }
  return paramList;
}

int getopt();
extern char *optarg;

int main (int argc, char **argv)
{
  int cod_arg;
  char *bufFloat;
  char outputMode;
  SxmlNode *paramMap;
  SxmlNode *inputDoc;
  SxmlNode *placeNode;

  traceLevel=0;
  outputMode='g';
  bufFloat=malloc(20);
  while ((cod_arg = getopt(argc,argv,"T:szg"))!=EOF)
    {switch(cod_arg) 
      {
      case 'T':
	traceLevel=atoi(optarg);
	break;
      case 's':
	outputMode='s';
	break;
      case 'z':
	outputMode='z';
	break;
      }
    }
  paramMap=NULL;
  while ((inputDoc=SxmlInputNextDocumentElement()))
    {
      SxmlNode *c1;
      SxmlNode *c2;
      SxmlNode *c3;
      SxmlNode *c4;

      char *c1str;
      char *c2str;
      char *c3str;


      SxmlNode *geoDilib;
      SxmlNode *svgPath;
      SxmlNode *svgI1;
      double xMin;
      double xMax;
      double yMin;
      double yMax;
      double xNew;
      double yNew;
      double xLen;
      double yLen;
      double scale;
      SxmlNode *svgArea;
      double upGeo;
      double leftGeo;
      double rightGeo;
      double downGeo;

      c1=SxmlGetFirstChildByTagName(inputDoc,"c1");
      if (c1)c1str=SxmlLeafText(c1);
      else continue;
      c2=SxmlGetFirstChildByTagName(inputDoc,"c2");
      c3=SxmlGetFirstChildByTagName(inputDoc,"c3");
      c4=SxmlGetFirstChildByTagName(inputDoc,"c4");

      if (strcmp(c1str, "place")==0)
	{
	  proceedPlace(paramMap, c2, c3, c4);
	  continue;
	}
     if (strcmp(c1str, "svg")==0)
	{
	  paramMap=proceedSvg(paramMap, c3);
	  continue;
	}
     if (strcmp(c1str, "point")==0)
	{
	  proceedPoint(paramMap, c3);
	  continue;
	}
     continue;
     
     if (c2)c2str=SxmlLeafText(c2);
     if (c3)c3str=SxmlLeafText(c3);
 
      svgI1=SxmlFirstChild(svgPath);
      xMin=SvgPathCommandX(svgI1);
      xMax=SvgPathCommandX(svgI1);
      yMin=SvgPathCommandY(svgI1);
      yMax=SvgPathCommandY(svgI1);
      if (traceLevel>1)
	{
	  printf("%s %3.3f %3.3f\n", SxmlNodeName(svgI1), SvgPathCommandX(svgI1), SvgPathCommandY(svgI1));
	}

      while ((svgI1=SxmlNextSibling(svgI1)))
	{
	  switch(*SxmlNodeName(svgI1))
	    {
	    case 'Z':
	    case 'z':
	      break;
	    case 'L':
	    case 'M':
	    case 'C':
	      xNew=SvgPathCommandX(svgI1);
	      yNew=SvgPathCommandY(svgI1);
	      if (xNew>xMax)xMax=xNew;
	      if (yNew>yMax)yMax=yNew;
	      if (xNew<xMin)xMin=xNew;
	      if (yNew<yMin)yMin=yNew;
	      if (traceLevel>1)
		{
		  printf("%s %3.3f %3.3f\n", SxmlNodeName(svgI1), SvgPathCommandX(svgI1), SvgPathCommandY(svgI1));
		}
	    }
	}
      if (traceLevel>0)
	{
	  printf (" %3.3f %3.3f  %3.3f %3.3f \n", xMin, yMin, xMax, yMax);
	}
      xLen=xMax-xMin;
      yLen=yMax-yMin;
      if (outputMode=='s')
	{
	  SxmlAppendChild(svgArea, SxmlLeafDoubleCreate( "up", yMax));
	  SxmlAppendChild(svgArea, SxmlLeafDoubleCreate( "left", xMin));
	  SxmlAppendChild(svgArea, SxmlLeafDoubleCreate( "down", yMin));
	  SxmlAppendChild(svgArea, SxmlLeafDoubleCreate( "right", xMax));
	  SxmlAppendChild(svgArea, SxmlLeafDoubleCreate( "width", xLen));
	  SxmlAppendChild(svgArea, SxmlLeafDoubleCreate( "height", yLen));
	  SxmlSave (placeNode);
	  SxmlFree(placeNode);
	  putchar('\n');
	  continue;
	}

      if (outputMode=='z')
	{
	  if (xLen<yLen)
	    {
	      scale=1000/xLen;
	      SxmlAppendChild(svgArea, SxmlLeafDoubleCreate( "width", (double)1000));
	      SxmlAppendChild(svgArea, SxmlLeafDoubleCreate( "height",  1000*yLen/xLen));
	    }
	  else
	    {
	      scale=1000/yLen;
	      SxmlAppendChild(svgArea, SxmlLeafDoubleCreate( "height", (double)1000));
	      SxmlAppendChild(svgArea, SxmlLeafDoubleCreate( "width",  1000*xLen/yLen));
	    }
	  svgI1=SxmlFirstChild(svgPath);
	  while (svgI1)
	    {
	      if (traceLevel>2) printf("%s ", SxmlNodeName(svgI1));
	      switch(*SxmlNodeName(svgI1))
		{
		case 'Z':
		case 'z':
		  break;
		case 'L':
		case 'M':
		  if (traceLevel>2) 
		    {
		      printf("%4.3f ",  SvgPathCommandX(svgI1));
		      printf("%4.3f - ",  SvgPathCommandY(svgI1));
		    }
		  SvgPathCommandSetX(svgI1, (SvgPathCommandX(svgI1)-xMin)*scale);
		  SvgPathCommandSetY(svgI1, (SvgPathCommandY(svgI1)-yMin)*scale);
		  if (traceLevel>2) 
		    {
		      printf("%4.3f ",  SvgPathCommandX(svgI1));
		      printf("%4.3f - ",  SvgPathCommandY(svgI1));
		    }
		  break;
		case 'C':
		  SvgPathCommandSetX(svgI1, (SvgPathCommandX(svgI1)-xMin)*scale);
		  SvgPathCommandSetY(svgI1, (SvgPathCommandY(svgI1)-yMin)*scale);
		  SvgPathCommandSetItem(svgI1, "x1", (SvgPathCommandItem(svgI1, "x1")-xMin)*scale);
		  SvgPathCommandSetItem(svgI1, "y1",  (SvgPathCommandItem(svgI1, "y1")-yMin)*scale);
		  SvgPathCommandSetItem(svgI1, "x2", (SvgPathCommandItem(svgI1, "x2")-xMin)*scale);
		  SvgPathCommandSetItem(svgI1, "y2", (SvgPathCommandItem(svgI1, "y2")-yMin)*scale);
		  if (traceLevel>2) 
		    {
		      printf("%4.3f ",  SvgPathCommandX(svgI1));
		      printf("%4.3f - ",  SvgPathCommandY(svgI1));
		      printf("%4.3f ",  (SvgPathCommandX(svgI1)-xMin)*scale);
		      printf("%4.3f\n",  (SvgPathCommandY(svgI1)-yMin)*scale);
		    }
		  break;
		}
	      if (traceLevel>1) { SxmlPrint(svgI1); putchar('\n');}
	      svgI1=SxmlNextSibling(svgI1);
	    }
	}
      else
	{
	  SxmlNode *path;
	  double scaleX;
	  double scaleY;

	  scaleX=(rightGeo-leftGeo)/xLen;
	  scaleY=(upGeo-downGeo)/yLen;

	  SxmlAppendChild(geoDilib, path=SxmlElementCreate("path"));
	  SxmlReset(svgPath);
	  while ((svgI1=SxmlNextNode(svgPath)))
	    {
	      SxmlNode *svgI2;
	      char *commandName;
	      commandName=SxmlNodeName(svgI1);
	      SxmlAppendChild(path, svgI2=SxmlElementCreate(commandName));
	      switch(*commandName)
		{
		case 'Z':
		case 'z':
		  break;
		case 'L':
		case 'M':
		  SxmlAppendChild(svgI2, SxmlLeafDoubleCreate("x", leftGeo+(SvgPathCommandX(svgI1)-xMin)*scaleX));
		  SxmlAppendChild(svgI2, SxmlLeafDoubleCreate("y", upGeo-(SvgPathCommandY(svgI1)-yMin)*scaleY));
		  break;
		case 'C':
		  SxmlAppendChild(svgI2, SxmlLeafDoubleCreate("x", leftGeo+(SvgPathCommandX(svgI1)-xMin)*scaleX));
		  SxmlAppendChild(svgI2, SxmlLeafDoubleCreate("y", upGeo-(SvgPathCommandY(svgI1)-yMin)*scaleY));
		  SxmlAppendChild(svgI2, SxmlLeafDoubleCreate("x1", leftGeo+(SvgPathCommandItem(svgI1, "x1")-xMin)*scaleX));
		  SxmlAppendChild(svgI2, SxmlLeafDoubleCreate("y1", upGeo-(SvgPathCommandItem(svgI1, "y1")-yMin)*scaleY));
		  SxmlAppendChild(svgI2, SxmlLeafDoubleCreate("x2", leftGeo+(SvgPathCommandItem(svgI1, "x2")-xMin)*scaleX));
		  SxmlAppendChild(svgI2, SxmlLeafDoubleCreate("y2", upGeo-(SvgPathCommandItem(svgI1, "y2")-yMin)*scaleY));
		}
	    }
	  SxmlFree(svgPath);
	  SxmlFree(svgArea);
	}
      SxmlSave (placeNode);
      SxmlFree(placeNode);
      putchar('\n');
    }
  exit(EXIT_SUCCESS);
}
