/*   -*- coding: utf-8 -*-  */

/******************************************************************************
*
*        Module   : Tei
*        Fichier  : TeiTableCityFromWiki
*        Auteur   : J. DUCLOY
*        Date     : janvier 2014
*
 ******************************************************************************/
#include <stdio.h>
#include "SxmlNode.h"
#include <getopt.h>
#include <stdlib.h>

SxmlNode *inputDoc;
SxmlNode *place;

void usage()
{
  fprintf(stderr,"usage: TeiTableCityFromWiki -g -e eTag -k kTag \n");
  exit(EXIT_FAILURE);
}

int getopt();
extern char *optarg;

 static  int  gOption;
 static  char *tagElem;
 static  char *tagKeys;
 static  char *prefix;

  SxmlNode *orgName;
  SxmlNode *country;
  SxmlNode *region;
  SxmlNode *placeName;
  SxmlNode *settlement;
  SxmlNode *inputSettlement;

void buildPlace()
{
  SxmlNode *keyElem;
  SxmlNode *listKey;
  SxmlNode *c1;
  SxmlNode *p1;
  SxmlNode *k1;
  SxmlNode *colElem;
  SxmlNode *typeSettlement;
  
  place=SxmlElementCreate("place");
  SxmlAppendChild(place, placeName=SxmlElementCreate("placeName"));
  
  listKey=SxmlGetFirstChildByTagName(inputDoc,tagKeys);
  c1=SxmlGetFirstChildByTagName(inputDoc,"c1");
  p1=SxmlGetFirstChildByTagName(c1,"p");
  k1=SxmlGetFirstChildByTagName(p1,"k");
  colElem=SxmlGetFirstChildByTagName(inputDoc,tagElem);
  country=SxmlGetFirstChildByTagName(colElem,"country");
  region=SxmlGetFirstChildByTagName(colElem,"region");
  typeSettlement=SxmlGetFirstChildByTagName(colElem,"typeSettlement");
  inputSettlement=SxmlGetFirstChildByTagName(colElem,"settlement");
  SxmlAppendChild(placeName, settlement=SxmlLeafCreate("settlement",  SxmlLeafText(k1)));
  if ((typeSettlement))
    SxmlSetAttribute(settlement, "type", SxmlLeafText(typeSettlement));
  else SxmlSetAttribute(settlement, "type", "city");
  if (country) SxmlAppendChild(place, SxmlClone(country));
  while(region)
    {
      SxmlAppendChild(placeName, SxmlClone(region));
      region=SxmlGetNextSiblingByTagName(region, "region");
    }
  while(inputSettlement)
    {
      SxmlAppendChild(placeName, SxmlClone(inputSettlement));
      inputSettlement=SxmlGetNextSiblingByTagName(inputSettlement, "settlement");
    }
  SxmlReset(listKey);
  while ((keyElem=SxmlNextNode(listKey)))
    {
      if (SxmlIsLeaf(keyElem))
	{
	  char *keyStr;
	  keyStr=SxmlLeafText(keyElem);
	  if (keyStr[0])
	    {
	      if (prefix) printf("%s", prefix);
	      printf ("%s\t", keyStr);
	      SxmlPrint(place);
	      putchar ('\n');
	    }
	}
    }
  SxmlFree(place);
}

SxmlNode *regionSwitch(SxmlNode *gNode)
{
  SxmlNode *evalChildNode;
  SxmlNode *switchNode;
  SxmlNode *lNode;
  SxmlNode *kNode;
  SxmlNode *giNode;

  evalChildNode=SxmlElementCreate ("dom:evalChild");
  SxmlSetAttribute(evalChildNode, "tag", "placeName");
  SxmlAppendChild(evalChildNode, switchNode=SxmlElementCreate ("dom:switch"));
  lNode=SxmlGetFirstChildByTagName(gNode, "l");
  while ((giNode=SxmlNextNode(lNode)))
    {
      SxmlNode *caseNode;
      kNode=SxmlGetFirstChildByTagName(giNode, "k");
      SxmlAppendChild(switchNode, caseNode=SxmlElementCreate("dom:hasChildLeaf")); 
      SxmlSetAttribute(caseNode, "tag", "region");
      SxmlSetAttribute(caseNode, "text", SxmlLeafText(kNode));
      SxmlAppendChild(caseNode, SxmlClone(SxmlGetFirstDescendantByTagName(giNode,"place")));
    }
  return(evalChildNode);
}

void buildPlaceSwitch()
{
  SxmlNode *tNode;
  SxmlNode *lNode;
  SxmlNode *kNode;
  SxmlNode *nNode;
  SxmlNode *giNode;
  SxmlNode *switchNode;
  tNode=SxmlGetFirstChildByTagName(inputDoc, "t");
  lNode=SxmlGetFirstChildByTagName(inputDoc, "l");
  kNode=SxmlGetFirstChildByTagName(inputDoc, "k");
  nNode=SxmlGetFirstChildByTagName(inputDoc, "n");
  printf("%s\t", SxmlLeafText(kNode));
  if (strcmp (SxmlLeafText(tNode),"1")==0)
    {
      SxmlPrint(SxmlGetFirstDescendantByTagName(inputDoc,"place"));
      putchar('\n');
      return;
    }
  if (strcmp (SxmlLeafText(nNode),"1")==0)
    {
    }
  switchNode=SxmlElementCreate ("dom:switch");
  SxmlReset(lNode);
  while ((giNode=SxmlNextNode(lNode)))
    {
      SxmlNode *caseNode;
      kNode=SxmlGetFirstChildByTagName(giNode, "k");
      nNode=SxmlGetFirstChildByTagName(giNode, "n");
      SxmlAppendChild(switchNode, caseNode=SxmlElementCreate("dom:hasChildLeaf")); 
      SxmlSetAttribute(caseNode, "tag", "country");
      SxmlSetAttribute(caseNode, "text", SxmlLeafText(kNode));
      if (strcmp (SxmlLeafText(nNode),"1")==0)
	{
	  SxmlAppendChild(caseNode, SxmlClone(SxmlGetFirstDescendantByTagName(inputDoc,"place")));
	}
      else
	{
	  SxmlAppendChild(caseNode, regionSwitch(giNode));
	}
    }
  SxmlPrint(switchNode);
  putchar ('\n');
}


int main (int argc, char **argv)
{
  int cOption;

  gOption=0;
  tagElem="c3";
  tagKeys="c2";
  prefix=NULL;

  while((cOption=getopt(argc,argv,"e:k:gp:"))!=EOF)
    {
      switch (cOption)
	{
	case 'e':
	  tagElem=optarg;
	  break;
	case 'k':
	  tagKeys=optarg;
	  break;
	case 'g':
	  gOption=1;
	  break;
	case 'p':
	  prefix=optarg;
	  break;
	default:
	  usage();
	  break;
	}
    }

  while ((inputDoc=SxmlInputNextDocumentElement()))
    {
      if (gOption==0)buildPlace();
      else buildPlaceSwitch();
    }
   exit(EXIT_SUCCESS);
}
