/*   -*- coding: utf-8 -*-  */

#include "Utf8Text.h"
#include "Utf8Converter.h"
#include "Except.h"
#include <ctype.h>

Utf8TextIterator* Utf8TextIteratorCreate()
{
  Utf8TextIterator *i1;
  if(!(i1=(Utf8TextIterator *)malloc(sizeof(Utf8TextIterator))))
    {
      ExceptSetError ("Uf8Text","00", "malloc failed","","",1);
    }
   return i1;
}

Utf8TextIterator *Utf8TextIteratorInit(Utf8TextIterator *i1, char *s1)
{
  i1->string=s1;
  return i1;
}


char *Utf8BufferCharCat(Buffer *b1, char *s1, char **s2)
{
  unsigned char c1;
  if (!s1)return NULL;
  c1=s1[0];
  if (c1=='\0') return NULL;
  if ((c1 & 0x80)==0)   /* ascii */
    {
      BufferCharCat(b1,c1);
      if (s2) *s2=s1+1;
      return BufferString(b1);
    }
  if ((c1 & 0x20)==0)
    {
      BufferCharCat(b1,c1);
      BufferCharCat(b1,s1[1]);
      if (s2) *s2=s1+2;
      return BufferString(b1);
    }   /* to be improved */
  return BufferString(b1);
}

Utf8Converter *Utf8TextConverter=NULL;

char *Utf8BufferLowerCharCat(Buffer *b1, char *s1, char **s2)
{
  char *cRes;
  char *s3;
  if(!Utf8TextConverter) Utf8TextConverter=Utf8ConverterCreate();
  Utf8ConverterSetString(Utf8TextConverter, s1);
  cRes= Utf8AlphaCharConvert (Utf8TextConverter, 'a');
  if (cRes)
    {
      BufferStrcat(b1, cRes);
      if (s2) *s2=Utf8TextConverter->currentPosition;
      return BufferString(b1);
    }
  else
    {
      cRes=Utf8BufferCharCat(b1,s1,&s3);
      if (s2) *s2=s3;
      return cRes;
    }
}

char *Utf8IsCharAlpha(char *s1)
{
  unsigned char c1;
  unsigned char c2;
  if (!s1)return NULL;
  c1=s1[0];
  if (c1=='\0') return NULL;
  if ((c1 & 0x80)==0)   /* ascii */
    {
      if(isalpha(c1)) return s1;
      else return NULL;
    }
  c2=s1[1];
  if (c2=='\0') return NULL;
  switch(c1)
    {
    case 0xC2:
      switch(c2)
	{
	case 0x9C: case 0x8C: return s1; /* "œ Œ */
	default: return NULL;
	}
      return NULL;
    case 0xC3:
      switch(c2)
	{
	case 0x87: case 0x97: return NULL; 
	default: return s1;
	}
    case 0xC4: return s1;
    }
  return NULL;
}

char *Utf8BufferAlphaStringCat(Buffer *b1, char *s1, char **s2)
{
  char *toParse;
  if (!s1) return NULL;
  toParse=s1;
  if (Utf8IsCharAlpha(toParse))
    {
      Utf8BufferCharCat(b1, toParse, &toParse);
      while (Utf8IsCharAlpha(toParse))
	{
	  Utf8BufferCharCat(b1, toParse, &toParse);
	}
      if (s2) *s2=toParse;
      return BufferString(b1);
    }
  else return NULL;
}

char *Utf8BufferLowerStringCat(Buffer *b1, char *s1, char **s2)
{
  char *toParse;
  if (!s1) return NULL;
  toParse=s1;
  if (Utf8IsCharAlpha(toParse))
    {
      Utf8BufferLowerCharCat(b1, toParse, &toParse);
      while (Utf8IsCharAlpha(toParse))
	{
	  Utf8BufferLowerCharCat(b1, toParse, &toParse);
	}
      if (s2) *s2=toParse;
      return BufferString(b1);
    }
  else return NULL;
}


char *Utf8JumpChar(char *s1)
{
  unsigned char c1;
  if (!s1)return NULL;
  c1=s1[0];
  if (c1=='\0') return NULL;
  if ((c1 & 0x80)==0) return s1+1;
  if ((c1 & 0x20)==0) return s1+2;
  if ((c1 & 0x10)==0) return s1+3;
  return s1+4;
}

int Utf8Length(char *s1)
{
  int len;
  int i;
  unsigned char c1;

  if (!s1)return 0;
  len=0;
  i=0;
  while ((c1=s1[i]))
    {
      len++;
      if ((c1 & 0x80)==0) {i+=1; continue;}
      if ((c1 & 0x20)==0) {i+=2; continue;}
      if ((c1 & 0x10)==0) {i+=3; continue;}
    }
  return len;
}

char *Utf8SeekAlpha(char *s1)
{
  char *s2;
  char *toParse;
  if (!s1) return NULL;
  toParse=s1;
  while (*toParse)
    {
      if ((s2=Utf8IsCharAlpha(toParse))) return s2;
      toParse=Utf8JumpChar(toParse);
    }
  return NULL;
}

char *Utf8GetAlphaString(char *s1, char **s2, int mode)
/*
         mode :
                O : no transformation
                l : result in lower case
 */
{
  static Buffer *bufAlpha=NULL;
  char *nextAlpha;
  if (!bufAlpha) bufAlpha=NewBuffer();
  else BufferReset(bufAlpha);
  nextAlpha=Utf8SeekAlpha(s1);
  if (!nextAlpha)return NULL;
  if (mode=='l')
    {
      Utf8BufferLowerStringCat(bufAlpha, nextAlpha , s2);
    }
  else Utf8BufferAlphaStringCat(bufAlpha, nextAlpha , s2);
  return BufferString(bufAlpha);
}


